from wger.nutrition.tests.plan import *
from wger.nutrition.tests.copy_plan import *
from wger.nutrition.tests.pdf import *

from wger.nutrition.tests.ingredient import *
from wger.nutrition.tests.ingredient_overview import *
from wger.nutrition.tests.meal import *



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

from django.core.urlresolvers import reverse

from wger.manager.models import TrainingSchedule

from wger.manager.tests.testcase import WorkoutManagerTestCase
from wger.manager.tests.testcase import WorkoutManagerDeleteTestCase
from wger.manager.tests.testcase import WorkoutManagerEditTestCase


class AddWorkoutTestCase(WorkoutManagerTestCase):
    '''
    Tests adding a Workout
    '''

    def create_workout(self, logged_in=False):
        '''
        Helper function to test creating workouts
        '''

        # Create a workout
        count_before = TrainingSchedule.objects.count()
        response = self.client.get(reverse('wger.manager.views.add'))
        count_after = TrainingSchedule.objects.count()

        # There is always a redirect
        self.assertEqual(response.status_code, 302)

        # Test creating workout
        if not logged_in:

            self.assertEqual(count_before, count_after)
            self.assertEqual(count_after, 3)
            self.assertTemplateUsed('login.html')

        else:
            self.assertGreater(count_after, count_before)
            self.assertTemplateUsed('workout/view.html')

        # Test accessing workout
        response = self.client.get(reverse('wger.manager.views.view_workout', kwargs={'id': 1}))

        if logged_in:
            workout = TrainingSchedule.objects.get(pk=1)
            self.assertEqual(response.context['workout'], workout)
            self.assertEqual(response.status_code, 200)
        else:
            self.assertEqual(response.status_code, 302)
            #workout = TrainingSchedule.objects.get(pk = 1)

    def test_create_workout_anonymous(self):
        '''
        Test creating a workout as anonymous user
        '''

        self.user_logout()
        self.create_workout()

    def test_create_workout_logged_in(self):
        '''
        Test creating a workout a logged in user
        '''

        self.user_login()
        self.create_workout(logged_in=True)
        self.user_logout()


class DeleteTestWorkoutTestCase(WorkoutManagerDeleteTestCase):
    '''
    Tests deleting a Workout
    '''

    object_class = TrainingSchedule
    url = 'workout-delete'
    pk = 3
    user_success = 'test'
    user_fail = 'admin'


class EditWorkoutTestCase(WorkoutManagerEditTestCase):
    '''
    Tests editing a Workout
    '''

    object_class = TrainingSchedule
    url = 'workout-edit'
    pk = 3
    user_success = 'test'
    user_fail = 'admin'
    data = {'comment': 'A new comment'}


class WorkoutOverviewTestCase(WorkoutManagerTestCase):
    '''
    Tests the workout overview
    '''

    def get_workout_overview(self, logged_in=False):
        '''
        Helper function to test the workout overview
        '''

        response = self.client.get(reverse('wger.manager.views.overview'))

        # Page exists
        if logged_in:
            self.assertEqual(response.status_code, 200)
            self.assertEqual(len(response.context['workouts']), 2)
        else:
            self.assertEqual(response.status_code, 302)

    def test_dashboard_anonymous(self):
        '''
        Test creating a workout as anonymous user
        '''

        self.get_workout_overview()

    def test_dashboard_logged_in(self):
        '''
        Test creating a workout a logged in user
        '''
        self.user_login()
        self.get_workout_overview(logged_in=True)



from django.conf.urls import patterns, url
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import login_required

from wger.nutrition.views import IngredientCreateView
from wger.nutrition.views import IngredientEditView
from wger.nutrition.views import IngredientDeleteView
from wger.nutrition.views import PlanDeleteView
from wger.nutrition.views import PlanEditView
from wger.nutrition.views import MealCreateView
from wger.nutrition.views import MealEditView


urlpatterns = patterns('wger.nutrition.views',
    url(r'^overview/$', 'overview'),

    # Plans
    url(r'^add/$', 'add'),
    url(r'^(?P<id>\d+)/view/$', 'view'),

    url(r'^(?P<pk>\d+)/copy/$',
        'copy',
        name='nutrition-copy'),

    url(r'^(?P<pk>\d+)/delete/$',
        login_required(PlanDeleteView.as_view()),
        name='nutrition-delete'),
    url(r'^(?P<pk>\d+)/edit/$',
        login_required(PlanEditView.as_view()),
        name='nutrition-edit'),
    url(r'^(?P<id>\d+)/pdf/$', 'export_pdf'),

    # Meals
    url(r'^(?P<plan_pk>\d+)/meal/add/$',
        login_required(MealCreateView.as_view()),
        name='meal-add'),
    url(r'^meal/(?P<pk>\d+)/edit/$',
        login_required(MealEditView.as_view()),
        name='meal-edit'),

    url(r'^(?P<id>\d+)/delete/meal/$', 'delete_meal'),

    # Meal items
    url(r'^(?P<id>\d+)/edit/meal/(?P<meal_id>\d+)/item/(?P<item_id>\w*)$', 'edit_meal_item'),
    url(r'^delete/meal/item/(?P<item_id>\d+)$', 'delete_meal_item'),

    # Ingredients
    url(r'^ingredient/(?P<pk>\d+)/delete/$',
        permission_required('exercises.change_exercise')(IngredientDeleteView.as_view()),
        name='ingredient-delete'),

    url(r'^ingredient/(?P<pk>\d+)/edit/$',
        permission_required('exercises.change_exercise')(IngredientEditView.as_view()),
        name='ingredient-edit'),

    url(r'^ingredient/add/$',
        permission_required('exercises.change_exercise')(IngredientCreateView.as_view()),
        name='ingredient-add'),

    url(r'^ingredient/overview/$', 'ingredient_overview'),
    url(r'^ingredient/(?P<id>\d+)/view/$', 'ingredient_view'),
    url(r'^ingredient/(?P<id>\d+)/view/(?P<slug>[-\w]+)/$', 'ingredient_view'),
    url(r'^ingredient/search/$', 'ingredient_search'),
)



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

import json

from django.core.urlresolvers import reverse

from wger.exercises.models import Exercise

from wger.manager.tests.testcase import WorkoutManagerTestCase
from wger.manager.tests.testcase import WorkoutManagerDeleteTestCase


class ExerciseIndexTestCase(WorkoutManagerTestCase):

    def exercise_index(self, editor=False):
        '''
        Tests the exercise overview page
        '''

        response = self.client.get(reverse('wger.exercises.views.exercise_overview'))

        # Page exists
        self.assertEqual(response.status_code, 200)

        # Correct tab is selected
        self.assertEqual(response.context['active_tab'], 'exercises')

        # Correct categories are shown
        category_1 = response.context['categories'][0]
        self.assertEqual(category_1.id, 2)
        self.assertEqual(category_1.name, "Another category")

        category_2 = response.context['categories'][1]
        self.assertEqual(category_2.id, 3)
        self.assertEqual(category_2.name, "Yet another category")

        # Correct exercises in the categories
        exercises_1 = category_1.exercise_set.all()
        exercise_1 = exercises_1[0]
        exercise_2 = exercises_1[1]
        self.assertEqual(exercise_1.id, 2)
        self.assertEqual(exercise_1.name, "A very cool exercise")

        self.assertEqual(exercise_2.id, 1)
        self.assertEqual(exercise_2.name, "An exercise")

        # Only authorized users see the edit links
        if editor:
            self.assertContains(response, 'Add new exercise')
            self.assertContains(response, 'Edit category')
            self.assertContains(response, 'Delete category')
            self.assertContains(response, 'Add category')
        else:
            self.assertNotContains(response, 'Add new exercise')
            self.assertNotContains(response, 'Edit category')
            self.assertNotContains(response, 'Delete category')
            self.assertNotContains(response, 'Add category')

    def test_exercise_index_editor(self):
        '''
        Tests the exercise overview page as a logged in user with editor rights
        '''

        self.user_login('admin')
        self.exercise_index(editor=True)

    def test_exercise_index_non_editor(self):
        '''
        Tests the exercise overview page as a logged in user without editor rights
        '''

        self.user_login('test')
        self.exercise_index(editor=False)

    def test_exercise_index_logged_out(self):
        '''
        Tests the exercise overview page as an anonymous (logged out) user
        '''

        self.exercise_index(editor=False)


class ExerciseDetailTestCase(WorkoutManagerTestCase):
    '''
    Tests the exercise details page
    '''

    def exercise_detail(self, editor=False):
        '''
        Tests the exercise details page
        '''

        response = self.client.get(reverse('wger.exercises.views.exercise_view', kwargs={'id': 1}))
        self.assertEqual(response.status_code, 200)

        # Correct tab is selected
        self.assertEqual(response.context['active_tab'], 'exercises')

        # Exercise loaded correct muscles
        exercise_1 = response.context['exercise']
        self.assertEqual(exercise_1.id, 1)

        muscles = exercise_1.muscles.all()
        muscle_1 = muscles[0]
        muscle_2 = muscles[1]

        self.assertEqual(muscle_1.id, 1)
        self.assertEqual(muscle_2.id, 2)

        # Only authorized users see the edit links
        if editor:
            self.assertContains(response, 'Edit')
            self.assertContains(response, 'Delete')
            self.assertContains(response, 'Add new comment')

        else:
            self.assertNotContains(response, 'Edit')
            self.assertNotContains(response, 'Delete')
            self.assertNotContains(response, 'Add new comment')

        # Ensure that non-existent exercises throw a 404.
        response = self.client.get(reverse('wger.exercises.views.exercise_view', kwargs={'id': 42}))
        self.assertEqual(response.status_code, 404)

    def test_exercise_detail_editor(self):
        '''
        Tests the exercise details page as a logged in user with editor rights
        '''

        self.user_login('admin')
        self.exercise_detail(editor=True)

    def test_exercise_detail_non_editor(self):
        '''
        Tests the exercise details page as a logged in user without editor rights
        '''

        self.user_login('test')
        self.exercise_detail(editor=False)

    def test_exercise_detail_logged_out(self):
        '''
        Tests the exercise details page as an anonymous (logged out) user
        '''

        self.exercise_detail(editor=False)


class ExercisesTestCase(WorkoutManagerTestCase):
    '''
    Exercise test case
    '''

    def add_exercise_user_fail(self):
        '''
        Helper function to test adding exercises by users that aren't authorized
        '''

        # Add an exercise
        count_before = Exercise.objects.count()
        response = self.client.post(reverse('exercise-add'),
                                    {'category': 2,
                                    'name': 'my test exercise',
                                    'muscles': [1, 2]})
        count_after = Exercise.objects.count()

        # Exercise was not added
        self.assertEqual(count_before, count_after)
        self.assertTrue('login' in response['location'])

    def test_add_exercise_user_no_rights(self):
        '''
        Tests adding an exercise with a user without enough rights to do this
        '''

        self.user_login('test')
        self.add_exercise_user_fail()
        self.user_logout()

    def test_add_exercise_no_user(self):
        '''
        Tests adding an exercise with a logged out (anonymous) user
        '''

        self.user_logout()
        self.add_exercise_user_fail()
        self.user_logout()

    def test_add_exercise_administrator_user(self):
        '''
        Tests adding/editing an exercise with a user with enough rights to do this
        '''

        # Log in as 'admin'
        self.user_login()

        # Add an exercise
        count_before = Exercise.objects.count()
        response = self.client.post(reverse('exercise-add'),
                                    {'category': 2,
                                    'name': 'my test exercise',
                                    'muscles': [1, 2]})
        count_after = Exercise.objects.count()
        self.assertEqual(response.status_code, 302)
        new_location = response['Location']
        self.assertEqual(count_before + 1, count_after, 'Exercise was not added')

        response = self.client.get(new_location)
        exercise_id = response.context['exercise'].id

        # Exercise was saved
        response = self.client.get(reverse('wger.exercises.views.exercise_view',
                                   kwargs={'id': exercise_id}))
        self.assertEqual(response.status_code, 200)

        # Navigation tab
        self.assertEqual(response.context['active_tab'], 'exercises')

        exercise_1 = Exercise.objects.get(pk=exercise_id)
        self.assertEqual(exercise_1.name, 'my test exercise')

        # Wrong category - adding
        response = self.client.post(reverse('exercise-add'),
                                    {'category': 111,
                                    'name': 'my test exercise',
                                    'muscles': [1, 2]})
        self.assertTrue(response.context['form'].errors['category'])

        # Wrong category - editing
        response = self.client.post(reverse('exercise-edit', kwargs={'pk': '1'}),
                                    {'category': 111,
                                    'name': 'my test exercise',
                                    'muscles': [1, 2]})
        self.assertTrue(response.context['form'].errors['category'])

        # No muscles - adding
        response = self.client.post(reverse('exercise-add'),
                                    {'category': 1,
                                    'name': 'my test exercise',
                                    'muscles': []})
        self.assertTrue(response.context['form'].errors['muscles'])

        # No muscles - editing
        response = self.client.post(reverse('exercise-edit', kwargs={'pk': '1'}),
                                    {'category': 1,
                                    'name': 'my test exercise',
                                    'muscles': []})
        self.assertTrue(response.context['form'].errors['muscles'])
        self.user_logout()

    def search_exercise(self, fail=True):
        '''
        Helper function to test searching for exercises
        '''

        # Search for exercises (1 hit, "A very cool exercise")
        response = self.client.get(reverse('wger.exercises.views.exercise_search') + '?term=cool')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['exercises']), 1)
        self.assertEqual(response.context['exercises'][0].name, 'A very cool exercise')

        kwargs = {'HTTP_X_REQUESTED_WITH': 'XMLHttpRequest'}

        # AJAX-Search for exercises (1 hit, "A very cool exercise")
        response = self.client.get(reverse('wger.exercises.views.exercise_search') +
                                   '?term=cool', **kwargs)
        self.assertEqual(response.status_code, 200)
        result = json.loads(response.content)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0]['value'], 'A very cool exercise')

    def test_search_exercise_anonymous(self):
        '''
        Test deleting an exercise by an anonymous user
        '''

        self.search_exercise()

    def test_search_exercise_unauthorized(self):
        '''
        Test deleting an exercise by an unauthorized user
        '''

        self.user_login('test')
        self.search_exercise()
        self.user_logout()

    def test_search_exercise_authorized(self):
        '''
        Test deleting an exercise by an authorized user
        '''

        self.user_login()
        self.search_exercise(fail=False)
        self.user_logout()


class DeleteExercisesTestCase(WorkoutManagerDeleteTestCase):
    '''
    Exercise test case
    '''

    object_class = Exercise
    url = 'exercise-delete'
    pk = 2
    user_success = 'admin'
    user_fail = 'test'



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

import logging

from django.core.urlresolvers import reverse

from wger.manager.models import Set

from wger.manager.tests.testcase import WorkoutManagerTestCase

logger = logging.getLogger('workout_manager.custom')


class SetDeleteTestCase(WorkoutManagerTestCase):
    '''
    Tests deleting a set from a workout
    '''

    def delete_set(self, fail=True):
        '''
        Helper function to test deleting a set from a workout
        '''

        # Fetch the overview page
        count_before = Set.objects.count()
        response = self.client.get(reverse('wger.manager.views.delete_set', kwargs={'id': 3,
                                           'day_id': 5,
                                           'set_id': 3}))
        count_after = Set.objects.count()

        if fail:
            self.assertTrue(response.status_code in (302, 403))
            self.assertEqual(count_before, count_after)
        else:
            self.assertEqual(response.status_code, 302)
            self.assertEqual(count_before - 1, count_after)
            self.assertRaises(Set.DoesNotExist, Set.objects.get, pk=3)

    def test_delete_set_anonymous(self):
        '''
        Tests deleting a set from a workout as an anonymous user
        '''

        self.delete_set(fail=True)

    def test_delete_set_owner(self):
        '''
        Tests deleting a set from a workout as the owner user
        '''

        self.user_login('admin')
        self.delete_set(fail=True)

    def test_delete_set_other(self):
        '''
        Tests deleting a set from a workout as a logged user not owning the data
        '''

        self.user_login('test')
        self.delete_set(fail=False)



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Workout Manager.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.core.urlresolvers import reverse

from wger.nutrition.models import Meal

from wger.manager.tests.testcase import WorkoutManagerDeleteTestCase
from wger.manager.tests.testcase import WorkoutManagerEditTestCase
from wger.manager.tests.testcase import WorkoutManagerAddTestCase


class EditMealTestCase(WorkoutManagerEditTestCase):
    '''
    Tests editing a meal
    '''

    object_class = Meal
    url = 'meal-edit'
    pk = 5
    data = {'time': datetime.time(8, 12)}


class AddMealTestCase(WorkoutManagerAddTestCase):
    '''
    Tests adding a Meal
    '''

    object_class = Meal
    url = reverse('meal-add', kwargs={'plan_pk': 4})
    pk = 12
    data = {'time': datetime.time(9, 2)}
    user_success = 'test'
    user_fail = 'admin'



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

import logging
import uuid
import datetime
import random


from django.template import RequestContext
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.http import HttpResponseForbidden
from django.core.context_processors import csrf
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy
from django.utils.formats import date_format

from django.forms.models import modelformset_factory


from django.contrib.auth import authenticate
from django.contrib.auth import login as django_login
from django.contrib.auth import logout as django_logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User as Django_User

from django.contrib.auth.views import login as django_loginview

from django.views.generic import DeleteView
from django.views.generic import CreateView
from django.views.generic import UpdateView
from django.views.generic import DetailView

from wger.manager.models import DaysOfWeek
from wger.manager.models import TrainingSchedule
from wger.manager.models import Day
from wger.manager.models import Set
from wger.manager.models import Setting
from wger.manager.models import WorkoutLog

from wger.exercises.models import Exercise

from wger.nutrition.models import NutritionPlan

from wger.weight.models import WeightEntry

from wger.manager.forms import UserPreferencesForm
from wger.manager.forms import UserEmailForm
from wger.manager.forms import RegistrationForm
from wger.manager.forms import DayForm
from wger.manager.forms import WorkoutForm
from wger.manager.forms import WorkoutCopyForm
from wger.manager.forms import SetForm
from wger.manager.forms import HelperDateForm
from wger.manager.forms import WorkoutLogForm
from wger.manager.forms import DemoUserForm

from wger.manager.utils import load_language

from wger.workout_manager.generic_views import YamlFormMixin
from wger.workout_manager.generic_views import YamlDeleteMixin


logger = logging.getLogger('workout_manager.custom')


# ************************
# Misc functions
# ************************
def index(request):
    if request.user.is_authenticated():
        return HttpResponseRedirect(reverse('dashboard'))
    else:
        return HttpResponseRedirect(reverse('software:features'))


@login_required
def dashboard(request):
    '''
    Show the index page, in our case, the last workout and nutritional plan
    and the current weight
    '''

    template_data = {}

    # Load the last workout, if one exists
    try:
        current_workout = TrainingSchedule.objects.filter(user=request.user).latest('creation_date')
    except ObjectDoesNotExist:
        current_workout = False
    template_data['current_workout'] = current_workout

    # Load the last nutritional plan, if one exists
    try:
        plan = NutritionPlan.objects.filter(user=request.user).latest('creation_date')
    except ObjectDoesNotExist:
        plan = False
    template_data['plan'] = plan

    # Load the last logged weight entry, if one exists
    try:
        weight = WeightEntry.objects.filter(user=request.user).latest('creation_date')
    except ObjectDoesNotExist:
        weight = False
    template_data['weight'] = weight

    if current_workout:
        # Format a bit the days so it doesn't have to be done in the template
        week_day_result = []
        for week in DaysOfWeek.objects.all():
            day_has_workout = False
            for day in current_workout.day_set.select_related():
                for day_of_week in day.day.select_related():
                    if day_of_week.id == week.id:
                        day_has_workout = True
                        week_day_result.append((_(week.day_of_week), day.description, True))
                        break

            if not day_has_workout:
                week_day_result.append((_(week.day_of_week), _('Rest day'), False))

        template_data['weekdays'] = week_day_result

    if plan:

        # Load the nutritional info
        template_data['nutritional_info'] = plan.get_nutritional_values()

    return render_to_response('index.html',
                              template_data,
                              context_instance=RequestContext(request))


# ************************
# User functions
# ************************
def login(request):
    '''
    Small wrapper around the django login view
    '''

    return django_loginview(request,
                            template_name='user/login.html')


def logout(request):
    '''
    Logout the user
    '''
    django_logout(request)
    return HttpResponseRedirect(reverse('login'))


def registration(request):
    '''
    A form to allow for registration of new users
    '''
    template_data = {}
    template_data.update(csrf(request))

    if request.method == 'POST':

        # If the user is already logged in, redirect
        if request.user.is_authenticated():
            return HttpResponseRedirect(reverse('dashboard'))

        form = RegistrationForm(data=request.POST)

        # If the data is valid, log in and redirect
        if form.is_valid():
            username = form.cleaned_data['username']
            password = form.cleaned_data['password1']
            email = form.cleaned_data['email']
            user = Django_User.objects.create_user(username,
                                                   email,
                                                   password)
            user.save()
            user = authenticate(username=username, password=password)
            django_login(request, user)
            return HttpResponseRedirect(reverse('dashboard'))
    else:
        form = RegistrationForm()

    template_data['form'] = form
    template_data['title'] = _('Register')
    template_data['form_fields'] = [i for i in form]
    template_data['form_action'] = reverse('wger.manager.views.registration')
    template_data['submit_text'] = _('Register')

    return render_to_response('form.html',
                              template_data,
                              context_instance=RequestContext(request))


def create_demo_user(request):
    '''
    Creates a demo user and adds some initial data
    '''

    template_data = {}
    template_data.update(csrf(request))

    if request.method == 'POST':
        form = DemoUserForm(data=request.POST)

        # If the data is valid, create a user, log in and redirect
        if form.is_valid():
            username = uuid.uuid4().hex[:-2]
            password = uuid.uuid4().hex[:-2]
            email = ''
            user = Django_User.objects.create_user(username,
                                                   email,
                                                   password)
            user.save()
            user_profile = user.get_profile()
            user_profile.is_temporary = True
            user_profile.save()

            #
            # Create some initial data
            # (this is a bit ugly...)
            #

            # Workout and exercises
            workout = TrainingSchedule(user=user,
                                       comment=_('Sample workout'))
            workout.save()
            monday = DaysOfWeek.objects.get(pk=1)
            wednesday = DaysOfWeek.objects.get(pk=3)
            day = Day(training=workout,
                      description=_('Sample day'))
            day.save()
            day.day.add(monday)

            day2 = Day(training=workout,
                       description=_('Another sample day'))
            day2.save()
            day2.day.add(wednesday)

            # Biceps curls with dumbbell
            if(load_language().short_name == 'de'):
                exercise = Exercise.objects.get(pk=4)
            else:
                exercise = Exercise.objects.get(pk=91)
            day_set = Set(exerciseday=day,
                          sets=4,
                          order=2)
            day_set.save()
            day_set.exercises.add(exercise)

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=8,
                              order=1)
            setting.save()

            # Weight log entries
            for reps in (7, 8, 9, 10):
                for i in range(1, 8):
                    log = WorkoutLog(user=user,
                                     exercise=exercise,
                                     workout=workout,
                                     reps=reps,
                                     weight=30 - reps + random.randint(1, 5),
                                     date=datetime.date.today() - datetime.timedelta(weeks=i))
                    log.save()

            # French press
            if(load_language().short_name == 'de'):
                exercise = Exercise.objects.get(pk=25)
            else:
                exercise = Exercise.objects.get(pk=84)
            day_set = Set(exerciseday=day,
                          sets=4,
                          order=2)
            day_set.save()
            day_set.exercises.add(exercise)

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=8,
                              order=1)
            setting.save()

            # Squats
            if(load_language().short_name == 'de'):
                exercise = Exercise.objects.get(pk=6)
            else:
                exercise = Exercise.objects.get(pk=111)
            day_set = Set(exerciseday=day,
                          sets=4,
                          order=3)
            day_set.save()
            day_set.exercises.add(exercise)

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=10,
                              order=1)
            setting.save()

            # Crunches
            if(load_language().short_name == 'de'):
                exercise = Exercise.objects.get(pk=26)
            else:
                exercise = Exercise.objects.get(pk=81)
            day_set = Set(exerciseday=day,
                          sets=4,
                          order=4)
            day_set.save()
            day_set.exercises.add(exercise)

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=30,
                              order=1)
            setting.save()

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=99,
                              order=2)
            setting.save()

            setting = Setting(set=day_set,
                              exercise=exercise,
                              reps=35,
                              order=3)
            setting.save()

            # (Body) weight entries
            for i in range(1, 20):
                creation_date = datetime.date.today() - datetime.timedelta(days=i)
                entry = WeightEntry(user=user,
                                    weight=80 + 0.5*i + random.randint(1, 3),
                                    creation_date=creation_date)
                entry.save()

            user = authenticate(username=username, password=password)
            django_login(request, user)
            return HttpResponseRedirect(reverse('dashboard'))
    else:
        form = DemoUserForm()

    template_data['form'] = form
    template_data['submit_text'] = _('Register')

    return render_to_response('user/demo.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def preferences(request):
    '''
    An overview of all user preferences
    '''
    template_data = {}
    template_data.update(csrf(request))
    redirect = False

    # Process the preferences form
    if request.method == 'POST':

        form = UserPreferencesForm(data=request.POST, instance=request.user.get_profile())
        form.user = request.user

        # Save the data if it validates
        if form.is_valid():
            form.save()
            redirect = True
    else:
        form = UserPreferencesForm(instance=request.user.get_profile())

    # Process the email form
    #
    # this is a bit ugly, but we need to take special care here, only instatiating
    # the form when the user changes its password, otherwise the email form will
    # check the adress for uniqueness and fail, because it will find one user (the
    # current one) using it. But if he changes it, we want to check that nobody else
    # has that email
    if request.method == 'POST' and request.POST["email"] != request.user.email:
        email_form = UserEmailForm(data=request.POST, instance=request.user)
        if email_form.is_valid():
            request.user.email = email_form.cleaned_data['email']
            request.user.save()
            redirect = True
    else:
        email_form = UserEmailForm(instance=request.user)

    template_data['form'] = form
    template_data['email_form'] = email_form

    if redirect:
        return HttpResponseRedirect(reverse('preferences'))
    else:
        return render_to_response('user/preferences.html',
                                  template_data,
                                  context_instance=RequestContext(request))


@login_required
def api_user_preferences(request):
    '''
    Allows the user to edit its preferences via AJAX calls
    '''

    if request.is_ajax():

        # Show comments on workout view
        if request.GET.get('do') == 'set_show-comments':
            new_value = int(request.GET.get('show'))

            profile = request.user.get_profile()
            profile.show_comments = new_value
            profile.save()

            return HttpResponse(_('Success'))

        # Show ingredients in english
        elif request.GET.get('do') == 'set_english-ingredients':
            new_value = int(request.GET.get('show'))

            profile = request.user.get_profile()
            profile.show_english_ingredients = new_value
            profile.save()

            return HttpResponse(_('Success'))


# ************************
# Workout functions
# ************************
@login_required
def overview(request):
    '''
    An overview of all the user's workouts
    '''

    template_data = {}

    latest_trainings = TrainingSchedule.objects.filter(user=request.user)
    template_data['workouts'] = latest_trainings

    return render_to_response('workout/overview.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def view_workout(request, id):
    '''
    Show the workout with the given ID
    '''
    template_data = {}

    workout = get_object_or_404(TrainingSchedule, pk=id, user=request.user)
    template_data['workout'] = workout

    # TODO: this can't be performant, see if it makes problems
    # Create the backgrounds that show what muscles the workout will work on
    backgrounds_front = []
    backgrounds_back = []
    for day in workout.day_set.select_related():
        for set in day.set_set.select_related():
            for exercise in set.exercises.select_related():
                for muscle in exercise.muscles.all():
                    muscle_bg = 'images/muscles/main/muscle-%s.svg' % muscle.id

                    # Add the muscles to the background list, but only once.
                    #
                    # While the combining effect (the more often a muscle gets
                    # added, the more intense the colour) is interesting, the
                    # end result is a very unnatural, very bright colour.
                    if muscle.is_front and muscle_bg not in backgrounds_front:
                        backgrounds_front.append(muscle_bg)
                    elif not muscle.is_front and muscle_bg not in backgrounds_back:
                        backgrounds_back.append(muscle_bg)

    # Append the correct "main" background, with the silhouette of the human body
    backgrounds_front.append('images/muscles/muscular_system_front.svg')
    backgrounds_back.append('images/muscles/muscular_system_back.svg')

    template_data['muscle_backgrounds_front'] = backgrounds_front
    template_data['muscle_backgrounds_back'] = backgrounds_back

    return render_to_response('workout/view.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def copy_workout(request, pk):
    '''
    Makes a copy of a workout
    '''

    workout = get_object_or_404(TrainingSchedule, pk=pk, user=request.user)

    # Process request
    if request.method == 'POST':
        workout_form = WorkoutCopyForm(request.POST)

        if workout_form.is_valid():

            # Copy workout
            days = workout.day_set.all()

            workout_copy = workout
            workout_copy.pk = None
            workout_copy.comment = workout_form.cleaned_data['comment']
            workout_copy.save()

            # Copy the days
            for day in days:
                sets = day.set_set.all()

                day_copy = day
                day_copy.pk = None
                day_copy.training = workout_copy
                day_copy.save()

                # Copy the sets
                for current_set in sets:
                    current_set_id = current_set.id
                    exercises = current_set.exercises.all()

                    current_set_copy = current_set
                    current_set_copy.pk = None
                    current_set_copy.exerciseday = day_copy
                    current_set_copy.save()

                    # Exercises has Many2Many relationship
                    current_set_copy.exercises = exercises

                    # Go through the exercises
                    for exercise in exercises:
                        settings = exercise.setting_set.filter(set_id=current_set_id)

                        # Copy the settings
                        for setting in settings:
                            setting_copy = setting
                            setting_copy.pk = None
                            setting_copy.set = current_set_copy
                            setting_copy.save()

            return HttpResponseRedirect(reverse('wger.manager.views.view_workout',
                                        kwargs={'id': workout.id}))
    else:
        workout_form = WorkoutCopyForm({'comment': workout.comment})

        template_data = {}
        template_data.update(csrf(request))
        template_data['title'] = _('Copy workout')
        template_data['form'] = workout_form
        template_data['form_action'] = reverse('workout-copy', kwargs={'pk': workout.id})
        template_data['form_fields'] = [workout_form['comment']]

        return render_to_response('form.html',
                                  template_data,
                                  context_instance=RequestContext(request))


@login_required
def add(request):
    '''
    Add a new workout and redirect to its page
    '''
    workout = TrainingSchedule()
    workout.user = request.user
    workout.save()

    return HttpResponseRedirect(reverse('wger.manager.views.view_workout',
                                        kwargs={'id': workout.id}))


class WorkoutDeleteView(YamlDeleteMixin, DeleteView):
    '''
    Generic view to delete a workout routine
    '''

    model = TrainingSchedule
    success_url = reverse_lazy('wger.manager.views.overview')
    title = ugettext_lazy('Delete workout')
    form_action_urlname = 'workout-delete'


class WorkoutEditView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing workout routine
    '''

    model = TrainingSchedule
    form_class = WorkoutForm
    title = ugettext_lazy('Edit workout')
    form_action_urlname = 'workout-edit'


# ************************
# Day functions
# ************************
class DayView(YamlFormMixin):
    '''
    Base generic view for exercise day
    '''

    model = Day
    form_class = DayForm

    def get_success_url(self):
        return reverse('wger.manager.views.view_workout', kwargs={'id': self.object.training_id})

    def get_form(self, form_class):
        '''
        Filter the days of the week that are alreeady used by other days
        '''

        # Get the form
        form = super(DayView, self).get_form(form_class)

        # Calculate the used days ('used' by other days in the same workout)
        if self.object:
            workout = self.object.training
        else:
            workout = TrainingSchedule.objects.get(pk=self.kwargs['workout_pk'])

        used_days = []
        for day in workout.day_set.all():
            for weekday in day.day.all():
                if not self.object or day.id != self.object.id:
                    used_days.append(weekday.id)
        used_days.sort()

        # Set the queryset for day
        form.fields['day'].queryset = DaysOfWeek.objects.exclude(id__in=used_days)

        return form


class DayEditView(DayView, UpdateView):
    '''
    Generic view to update an existing exercise day
    '''

    title = ugettext_lazy('Edit workout day')
    form_action_urlname = 'day-edit'


class DayCreateView(DayView, CreateView):
    '''
    Generic view to add a new exercise day
    '''

    title = ugettext_lazy('Add workout day')
    owner_object = {'pk': 'workout_pk', 'class': TrainingSchedule}

    def form_valid(self, form):
        '''
        Set the workout this day belongs to
        '''
        form.instance.training = TrainingSchedule.objects.get(pk=self.kwargs['workout_pk'])
        return super(DayCreateView, self).form_valid(form)

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(DayCreateView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('day-add',
                                         kwargs={'workout_pk': self.kwargs['workout_pk']})
        return context


@login_required
def delete_day(request, id, day_id):
    '''
    Deletes the day with ID day_id belonging to workout with ID id
    '''

    # Load the day
    day = get_object_or_404(Day, pk=day_id)

    # Check if the user is the owner of the object
    if day.training.user == request.user:
        day.delete()
        return HttpResponseRedirect(reverse('wger.manager.views.view_workout', kwargs={'id': id}))
    else:
        return HttpResponseForbidden()


@login_required
def view_day(request, id):
    '''
    Renders a day as shown in the workout overview.

    This function is to be used with AJAX calls.
    '''
    template_data = {}

    # Load day and check if its workout belongs to the user
    day = get_object_or_404(Day, pk=id)
    if day.training.user != request.user:
        return HttpResponseForbidden()

    template_data['day'] = day

    return render_to_response('day/view.html',
                              template_data,
                              context_instance=RequestContext(request))


# ************************
# Set functions
# ************************
@login_required
def edit_set(request, id, day_id, set_id=None):
    '''
    Edits/creates a set
    '''

    template_data = {}
    template_data.update(csrf(request))

    # Load workout
    workout = get_object_or_404(TrainingSchedule, pk=id, user=request.user)
    template_data['workout'] = workout

    # Load day
    day = get_object_or_404(Day, pk=day_id)
    template_data['day'] = day

    # Load set

    # If the object is new, we will receice a 'None' (string) as the ID
    # from the template, so we check for it (ValueError) and for an actual
    # None (TypeError)
    try:
        int(set_id)
        workout_set = get_object_or_404(Set, pk=set_id)

        # Check if all objects belong to the workout
        if workout_set.exerciseday.id != day.id:
            return HttpResponseForbidden()
    except ValueError, TypeError:
        workout_set = Set()

    template_data['set'] = workout_set

    # Check if all objects belong to the workout
    if day.training.id != workout.id:
        return HttpResponseForbidden()

    # Process request
    if request.method == 'POST':

        set_form = SetForm(request.POST, instance=workout_set)

        # If the data is valid, save and redirect
        if set_form.is_valid():
            workout_set = set_form.save(commit=False)
            workout_set.exerciseday = day
            workout_set.save()

            # The exercises are ManyToMany in DB, so we have to save with this function
            set_form.save_m2m()

            return HttpResponseRedirect(reverse('wger.manager.views.view_workout',
                                                kwargs={'id': id}))
    else:
        set_form = SetForm(instance=workout_set)
    template_data['set_form'] = set_form

    return render_to_response('set/edit.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def delete_set(request, id, day_id, set_id):
    '''
    Deletes the given set
    '''

    # Load the set
    set_obj = get_object_or_404(Set, pk=set_id)

    # Check if the user is the owner of the object
    if set_obj.exerciseday.training.user == request.user:
        set_obj.delete()
        return HttpResponseRedirect(reverse('wger.manager.views.view_workout', kwargs={'id': id}))
    else:
        return HttpResponseForbidden()


@login_required
def api_edit_set(request):
    '''
    Allows to edit the order of the sets via an AJAX call
    '''

    if request.is_ajax():

        # Set the order of the reps
        if request.GET.get('do') == 'set_order':
            day_id = request.GET.get('day_id')
            new_set_order = request.GET.get('order')

            order = 0
            for i in new_set_order.strip(',').split(','):
                set_id = i.split('-')[1]
                order += 1

                set_obj = get_object_or_404(Set, pk=set_id, exerciseday=day_id)

                # Check if the user is the owner of the object
                if set_obj.exerciseday.training.user == request.user:
                    set_obj.order = order
                    set_obj.save()
                else:
                    return HttpResponseForbidden()

            return HttpResponse(_('Success'))

        # This part is responsible for the in-place editing of the sets and settings
        if request.GET.get('do') == 'edit_set':
            template_data = {}
            template_data.update(csrf(request))

            # Load the objects
            set_id = request.GET.get('set')
            workout_set = get_object_or_404(Set, pk=set_id)
            template_data['set'] = workout_set

            exercise_id = request.GET.get('exercise')
            exercise = get_object_or_404(Exercise, pk=exercise_id)

            # Allow editing settings/repetitions that are not yet associated with the set
            #
            # We calculate here how many are there already [.filter(...)] and how many there could
            # be at all (workout_set.sets)
            current_settings = exercise.setting_set.filter(set_id=set_id).count()
            diff = int(workout_set.sets) - current_settings

            # If there are 'free slots', create some UUIDs for them, this gives them unique form
            # names in the HTML and makes our lifes easier
            new_settings = []
            if diff > 0:

                # Note: use UUIDs version 1 because they are monotonously increasing
                #       and the order of the fields later is important
                new_settings = [uuid.uuid1() for i in range(0, diff)]
            template_data['new_settings'] = new_settings

            # Process request
            if request.method == 'POST':

                new_exercise_id = request.POST.get('current_exercise')
                new_exercise = get_object_or_404(Exercise, pk=new_exercise_id)

                # When there is more than one exercise per set, we need to manually set and replace
                # the IDs here, otherwise they get lost
                request_copy = request.POST
                request_copy = request_copy.copy()

                exercise_list = [i for i in request_copy.getlist('exercises') if i != exercise_id]
                request_copy.setlist('exercises', exercise_list)
                request_copy.update({'exercises': new_exercise_id})

                set_form = SetForm(request_copy, instance=workout_set)

                if set_form.is_valid():
                    set_form.save()

                # Init a counter for the order in case we have to set it for new settings
                # We don't actually care how hight the counter actually is, as long as the new
                # settings get a number that puts them at the end
                order_counter = 1
                new_settings = []

                # input fields for settings  'setting-x, setting-y, etc.',
                #              new settings: 'new-setting-UUID1, new-setting-UUID2, etc.'
                for i in request.POST:
                    order_counter += 1

                    # old settings, update
                    if i.startswith('setting'):
                        setting_id = int(i.split('-')[-1])
                        setting = get_object_or_404(Setting, pk=setting_id)

                        # Check if the new value is empty (the user wants the setting deleted)
                        # We don't check more, if the user enters a string, it won't be converted
                        # and nothing will happen
                        if request.POST[i] == '':
                            setting.delete()
                        else:
                            reps = int(request.POST[i])
                            setting.reps = reps
                            setting.exercise = new_exercise
                            setting.save()

                    # New settings, put in a list, see below
                    if i.startswith('new-setting') and request.POST[i]:

                        new_settings.append(i)

                # new settings, sort by name (important to keep the order as
                # it was inputted in the website),create object and save
                new_settings.sort()
                for i in new_settings:
                    reps = int(request.POST[i])

                    setting = Setting()
                    setting.exercise = new_exercise
                    setting.set = workout_set
                    setting.reps = reps
                    setting.order = order_counter
                    setting.save()

            template_data['exercise'] = exercise

            return render_to_response('setting/ajax_edit.html',
                                      template_data,
                                      context_instance=RequestContext(request))


# ************************
# Settings functions
# ************************
@login_required
def edit_setting(request, id, set_id, exercise_id, setting_id=None):
    template_data = {}
    template_data.update(csrf(request))

    # Load workout
    workout = get_object_or_404(TrainingSchedule, pk=id, user=request.user)
    template_data['workout'] = workout

    # Load set and the FormSet
    set_obj = get_object_or_404(Set, pk=set_id)
    template_data['set'] = set_obj

    SettingFormSet = modelformset_factory(Setting,
                                          exclude=('set', 'exercise'),
                                          max_num=int(set_obj.sets),
                                          extra=int(set_obj.sets))

    # Load exercise
    exercise = get_object_or_404(Exercise, pk=exercise_id)
    template_data['exercise'] = exercise

    # Check that the set belongs to the workout
    if set_obj.exerciseday.training.id != workout.id:
        return HttpResponseForbidden()

    # Load setting
    if not setting_id:
        setting = Setting()
    else:
        setting = get_object_or_404(Setting, pk=setting_id)
    template_data['setting'] = setting

    # Process request
    if request.method == 'POST':

        # Process the FormSet, setting the set and the exercise
        setting_form = SettingFormSet(request.POST)
        if setting_form.is_valid():

            order = 1
            instances = setting_form.save(commit=False)
            for setting_instance in instances:
                setting_instance.set = set_obj
                setting_instance.exercise = exercise

                # Manualy set the order, the user can later use drag&drop to change this
                if not setting_instance.order:
                    setting_instance.order = order

                setting_instance.save()

                order += 1

            return HttpResponseRedirect(reverse('wger.manager.views.view_workout',
                                                kwargs={'id': id}))
    else:
        setting_form = SettingFormSet(queryset=Setting.objects.filter(exercise_id=exercise.id,
                                                                      set_id=set_obj.id))
    template_data['setting_form'] = setting_form

    return render_to_response('setting/edit.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def api_edit_setting(request):
    '''
    Allows to edit the order of the setting inside a set via an AJAX call
    '''

    if request.is_ajax():
        if request.GET.get('do') == 'set_order':
            new_setting_order = request.GET.get('order')

            order = 0
            for i in new_setting_order.strip(',').split(','):
                setting_id = i.split('-')[1]
                order += 1

                setting_obj = get_object_or_404(Setting, pk=setting_id)

                # Check if the user is the owner of the object
                if setting_obj.set.exerciseday.training.user == request.user:
                    setting_obj.order = order
                    setting_obj.save()
                else:
                    return HttpResponseForbidden()

            return HttpResponse(_('Success'))


@login_required
def delete_setting(request, id, set_id, exercise_id):
    '''
    Deletes all the settings belonging to set_id and exercise_id
    '''

    # Load the workout
    workout = get_object_or_404(TrainingSchedule, pk=id, user=request.user)

    # Delete all settings
    settings = Setting.objects.filter(exercise_id=exercise_id, set_id=set_id)
    settings.delete()

    return HttpResponseRedirect(reverse('wger.manager.views.view_workout', kwargs={'id': id}))


# ************************
# Log functions
# ************************
class WorkoutLogUpdateView(YamlFormMixin, UpdateView):
    '''
    Generic view to edit an existing workout log weight entry
    '''
    model = WorkoutLog
    form_class = WorkoutLogForm
    custom_js = '''$(document).ready(function () {
        init_weight_log_datepicker();
    });'''

    def get_context_data(self, **kwargs):
        context = super(WorkoutLogUpdateView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('workout-log-edit', kwargs={'pk': self.object.id})
        context['title'] = _('Edit log entry for %s') % self.object.exercise.name

        return context

    def get_success_url(self):
        return reverse('workout-log', kwargs={'pk': self.object.workout_id})


def workout_log_add(request, pk):
    '''
    Add a new workout log
    '''

    template_data = {}
    template_data.update(csrf(request))

    # Load the day and check ownership
    day = get_object_or_404(Day, pk=pk)
    if day.get_owner_object().user != request.user:
        return HttpResponseForbidden()

    # We need several lists here because we need to assign specific form to each
    # exercise: the entries for weight and repetitions have no indicator to which
    # exercise they belong besides the form-ID, from Django's formset
    counter = 0
    total_sets = 0
    exercise_list = {}
    form_to_exercise = {}

    for exercise_set in day.set_set.all():
        for exercise in exercise_set.exercises.all():

            # Maximum possible values
            total_sets = total_sets + int(exercise_set.sets)
            counter_before = counter
            counter = counter + int(exercise_set.sets) - 1
            form_id_range = range(counter_before, counter + 1)

            # Add to list
            exercise_list[exercise.id] = {'obj': exercise,
                                          'sets': int(exercise_set.sets),
                                          'form_ids': form_id_range}

            counter = counter + 1
            # Helper mapping form-ID <--> Exercise
            for id in form_id_range:
                form_to_exercise[id] = exercise

    # Define the formset here because now we know the value to pass to 'extra'
    WorkoutLogFormSet = modelformset_factory(WorkoutLog,
                                             form=WorkoutLogForm,
                                             exclude=('user',
                                                      'workout',
                                                      'date'),
                                             extra = total_sets)
    # Process the request
    if request.method == 'POST':

        # Make a copy of the POST data and go through it. The reason for this is
        # that the form expects a value for the exercise which is not present in
        # the form (for space and usability reasons)

        post_copy = request.POST.copy()

        for form_id in form_to_exercise:
            if post_copy.get('form-%s-weight' % form_id) or post_copy.get('form-%s-reps' % form_id):
                post_copy['form-%s-exercise' % form_id] = form_to_exercise[form_id].id

        # Pass the new data to the forms
        formset = WorkoutLogFormSet(data=post_copy)
        dateform = HelperDateForm(data=post_copy)

        if dateform.is_valid():
            log_date = dateform.cleaned_data['date']

            # If the data is valid, save and redirect to log overview page
            if formset.is_valid():
                instances = formset.save(commit=False)
                for instance in instances:

                    instance.user = request.user
                    instance.workout = day.training
                    instance.date = log_date
                    instance.save()

                return HttpResponseRedirect(reverse('workout-log', kwargs={'pk': day.training_id}))
    else:
        # Initialise the formset with a queryset that won't return any objects
        # (we only add new logs here and that seems to be the fastest way)
        formset = WorkoutLogFormSet(queryset=WorkoutLog.objects.filter(exercise=-1))

        formatted_date = date_format(datetime.date.today(), "SHORT_DATE_FORMAT")
        dateform = HelperDateForm(initial={'date': formatted_date})

    # Pass the correct forms to the exercise list
    for exercise in exercise_list:

        form_id_from = min(exercise_list[exercise]['form_ids'])
        form_id_to = max(exercise_list[exercise]['form_ids'])
        exercise_list[exercise]['forms'] = formset[form_id_from:form_id_to + 1]

    template_data['day'] = day
    template_data['exercises'] = exercise_list
    template_data['formset'] = formset
    template_data['dateform'] = dateform
    template_data['form_action'] = reverse('day-log', kwargs={'pk': pk})

    return render_to_response('day/log.html',
                              template_data,
                              context_instance=RequestContext(request))


class WorkoutLogDetailView(DetailView):
    '''
    An overview of the workout's log
    '''

    model = TrainingSchedule
    template_name = 'workout/log.html'
    context_object_name = 'workout'

    def get_context_data(self, **kwargs):

        # Call the base implementation first to get a context
        context = super(WorkoutLogDetailView, self).get_context_data(**kwargs)

        # Prepare the entries for rendering and the D3 chart
        workout_log = {}
        entry = WorkoutLog()

        for day in self.object.day_set.select_related():
            workout_log[day] = {}

            for set in day.set_set.select_related():
                exercise_log = {}

                for exercise in set.exercises.select_related():
                    exercise_log[exercise] = []
                    logs = exercise.workoutlog_set.filter(user=self.request.user)
                    entry_log, chart_data = entry.process_log_entries(logs)
                    if entry_log:
                        exercise_log[exercise].append(entry_log)

                    if exercise_log:
                        workout_log[day][exercise] = {}
                        workout_log[day][exercise]['log_by_date'] = entry_log
                        workout_log[day][exercise]['div_uuid'] = str(uuid.uuid4())
                        workout_log[day][exercise]['chart_data'] = chart_data

        context['workout_log'] = workout_log
        context['reps'] = _("Reps")

        return context

    def dispatch(self, request, *args, **kwargs):
        '''
        Check for ownership
        '''

        workout = TrainingSchedule.objects.get(pk=kwargs['pk'])
        if workout.user != request.user:
            return HttpResponseForbidden()

        # Dispatch normally
        return super(WorkoutLogDetailView, self).dispatch(request, *args, **kwargs)



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Workout Manager.  If not, see <http://www.gnu.org/licenses/>.

import logging
import json

from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _
from django.utils.datastructures import SortedDict
from django.contrib.auth.models import User
from django.core.validators import MaxValueValidator
from django.core.urlresolvers import reverse


from wger.exercises.models import Exercise

from wger.workout_manager.helpers import DecimalJsonEncoder

logger = logging.getLogger('workout_manager.custom')


class TrainingSchedule(models.Model):
    '''
    Model for a training schedule
    '''

    # Metaclass to set some other properties
    class Meta:
        ordering = ["-creation_date", ]

    creation_date = models.DateField(_('Creation date'), auto_now_add=True)
    comment = models.TextField(verbose_name=_('Description'),
                               max_length=100,
                               blank=True,
                               help_text=_('''A short description or goal of the workout.
For example 'Focus on back' or 'Week 1 of program xy'.'''))
    user = models.ForeignKey(User, verbose_name=_('User'))

    def get_absolute_url(self):
        '''
        Returns the canonical URL to view a workout
        '''
        return reverse('wger.manager.views.view_workout', kwargs={'id': self.id})

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return str(self.creation_date)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self


class DaysOfWeek(models.Model):
    '''
    Model for the days of the week

    This model is needed so that 'Day' can have multiple days of the week selected
    '''

    day_of_week = models.CharField(max_length=9,
                                   verbose_name=_('Day of the week'))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return self.day_of_week


class Day(models.Model):
    '''
    Model for a training day
    '''

    training = models.ForeignKey(TrainingSchedule, verbose_name=_('Training'))
    description = models.CharField(max_length=100,
                                   verbose_name=_('Description'),
                                   help_text=_('Ususally a description about what parts are '
                                               'trained, like "Arms" or "Pull Day"'))
    day = models.ManyToManyField(DaysOfWeek,
                                 verbose_name=_('Day'))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "%s for TP %s" % (self.description, unicode(self.training))

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self.training


class Set(models.Model):
    '''
    Model for a set of exercises
    '''

    exerciseday = models.ForeignKey(Day, verbose_name=_('Exercise day'))
    exercises = models.ManyToManyField(Exercise, verbose_name=_('Exercises'))
    order = models.IntegerField(max_length=1,
                                blank=True,
                                null=True,
                                verbose_name=_('Order'))
    sets = models.IntegerField(validators=[MaxValueValidator(6)], verbose_name=_('Sets'))

    # Metaclass to set some other properties
    class Meta:
        ordering = ["order", ]

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "Set-ID %s" % (self.id)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self.exerciseday.training


class Setting(models.Model):
    '''
    Settings for an exercise (weight, reps, etc.)
    '''

    set = models.ForeignKey(Set, verbose_name=_('Sets'))
    exercise = models.ForeignKey(Exercise, verbose_name=_('Exercises'))
    reps = models.IntegerField(validators=[MaxValueValidator(40)], verbose_name=_('Repetitions'))
    order = models.IntegerField(blank=True, verbose_name=_('Repetitions'))
    comment = models.CharField(max_length=100, blank=True, verbose_name=_('Comment'))

    # Metaclass to set some other properties
    class Meta:
        ordering = ["order", "id"]

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "settings for exercise %s in set %s" % (self.exercise.id, self.set.id)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self.set.exerciseday.training


class WorkoutLog(models.Model):
    '''
    A log entry for an exercise
    '''

    user = models.ForeignKey(User, verbose_name=_('User'))
    exercise = models.ForeignKey(Exercise, verbose_name=_('Exercise'))
    workout = models.ForeignKey(TrainingSchedule, verbose_name=_('Workout'))

    reps = models.IntegerField(verbose_name=_('Repetitions'))
    weight = models.DecimalField(decimal_places=2,
                                 max_digits=5,
                                 verbose_name=_('Weight'))
    date = models.DateField(verbose_name=_('Date'))

    # Metaclass to set some other properties
    class Meta:
        ordering = ["date", "reps"]

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "Log entry: %s - %s kg on %s" % (self.reps,
                                                self.weight,
                                                self.date)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self

    def process_log_entries(self, logs):
        '''
        Processes and regroups a list of log entries so they can be rendered
        and passed to the D3 library to render a chart
        '''

        reps = []
        entry_log = SortedDict()
        chart_data = []
        max_weight = {}

        # Group by date
        for entry in logs:
            if entry.reps not in reps:
                reps.append(entry.reps)

            if not entry_log.get(entry.date):
                entry_log[entry.date] = []
            entry_log[entry.date].append(entry)

            # Find the maximum weight per date per repetition.
            # If on a day there are several entries with the same number of
            # repetitions, but different weights, only the entry with the
            # higher weight is shown in the chart
            if not max_weight.get(entry.date):
                max_weight[entry.date] = {entry.reps: entry.weight}

            if not max_weight[entry.date].get(entry.reps):
                max_weight[entry.date][entry.reps] = entry.weight

            if entry.weight > max_weight[entry.date][entry.reps]:
                max_weight[entry.date][entry.reps] = entry.weight

        # Group by repetitions
        for entry in logs:
            temp = {'date': '%s' % entry.date,
                    'id': 'workout-log-%s' % entry.id}

            for rep in reps:
                if entry.reps == rep:

                    # Only add if entry is the maximum for the day
                    if entry.weight == max_weight[entry.date][entry.reps]:
                        temp[rep] = entry.weight
                else:
                    temp[rep] = 0
            chart_data.append(temp)

        return (entry_log, json.dumps(chart_data, cls=DecimalJsonEncoder))


class UserProfile(models.Model):
    # This field is required.
    user = models.OneToOneField(User)

    # Flag to mark a temporary user (demo account)
    is_temporary = models.BooleanField(default=False)

    #
    # User preferences
    #

    # Show exercise comments on workout view
    show_comments = models.BooleanField(verbose_name=_('Show exercise comments'),
                                        help_text=_('Check to show exercise comments on the '
                                                    'workout view'))

    # Also show ingredients in english while composing a nutritional plan
    # (obviously this is only meaningful if the user has a language other than english)
    show_english_ingredients = models.BooleanField(
        verbose_name=_('Also use ingredients in English'),
        help_text=_('''Check to also show ingredients in English while creating
a nutritional plan. These ingredients are extracted from a list provided
by the US Department of Agriculture. It is extremely complete, with around
7000 entries, but can be somewhat overwhelming and make the search difficult.'''))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "Profile for user %s" % (self.user)


# Every new user gets a profile
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        UserProfile.objects.create(user=instance)

post_save.connect(create_user_profile, sender=User)



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License


from django.core.urlresolvers import reverse

from wger.exercises.models import Exercise

from wger.manager.tests.testcase import WorkoutManagerTestCase


class ExercisecommentsTestCase(WorkoutManagerTestCase):

    def exercise_add_comment(self, fail=True):
        '''
        Tests the exercise comments (fails because of permissions)
        '''

        # Load the exercise
        exercise_1 = Exercise.objects.get(pk=1)

        # Comments are loaded
        comments = exercise_1.exercisecomment_set.all()
        comment_1 = comments[0]
        self.assertEqual(comment_1.id, 1)
        self.assertEqual(comment_1.comment, "test 123")
        self.assertEqual(len(comments), 1)

        # Post a comment
        response = self.client.post(reverse('exercisecomment-add', kwargs={'exercise_pk': 1}),
                                    {'comment': 'a new cool comment'})
        comments = exercise_1.exercisecomment_set.all()

        self.assertEqual(response.status_code, 302)
        if fail:
            comments = exercise_1.exercisecomment_set.all()
            self.assertEqual(len(comments), 1)

        else:
            self.assertEqual(len(comments), 2)

        # Post an empty comment and check it doesn't get added
        response = self.client.post(reverse('exercisecomment-add', kwargs={'exercise_pk': 1}),
                                    {'comment': ''})
        comments = exercise_1.exercisecomment_set.all()

        if fail:
            self.assertEqual(len(comments), 1)
        else:
            self.assertEqual(response.status_code, 200)
            self.assertEqual(len(comments), 2)

    def test_exercise_add_comment_no_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login('test')
        self.exercise_add_comment(fail=True)
        self.user_logout()

    def test_exercise_add_comment_not_logged_in(self):
        '''
        Tests the exercise comments
        '''

        self.exercise_add_comment(fail=True)

    def test_exercise_add_comment_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login()
        self.exercise_add_comment(fail=False)

    def exercise_delete_comment(self, fail=True):
        '''
        Tests the deletion of exercise comments
        '''

        # Load the exercise
        exercise_1 = Exercise.objects.get(pk=1)

        # Comments are loaded
        comments = exercise_1.exercisecomment_set.all()
        comment_1 = comments[0]
        self.assertEqual(comment_1.id, 1)
        self.assertEqual(comment_1.comment, "test 123")
        self.assertEqual(len(comments), 1)

        # Delete the comment
        response = self.client.post(reverse('exercisecomment-delete', kwargs={'id': 1}))
        comments = exercise_1.exercisecomment_set.all()

        self.assertEqual(response.status_code, 302)
        if fail:
            comments = exercise_1.exercisecomment_set.all()
            self.assertEqual(len(comments), 1)

        else:
            self.assertEqual(len(comments), 0)

    def test_exercise_delete_comment_no_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login('test')
        self.exercise_delete_comment(fail=True)
        self.user_logout()

    def test_exercise_delete_comment_not_logged_in(self):
        '''
        Tests the exercise comments
        '''

        self.exercise_delete_comment(fail=True)

    def test_exercise_delete_comment_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login()
        self.exercise_delete_comment(fail=False)

    def exercise_edit_comment(self, fail=True):
        '''
        Tests the exercise comments (fails because of permissions)
        '''

        # Load the exercise
        exercise_1 = Exercise.objects.get(pk=1)
        comment_before = 'test 123'
        comment_after = 'a new cool comment'

        # Comments are loaded
        comments = exercise_1.exercisecomment_set.all()
        comment_1 = comments[0]
        self.assertEqual(comment_1.id, 1)
        self.assertEqual(comment_1.comment, comment_before)
        self.assertEqual(len(comments), 1)

        # Open the edit page
        response = self.client.get(reverse('exercisecomment-edit', kwargs={'pk': 1}))
        if fail:
            self.assertEqual(response.status_code, 302)
        else:
            self.assertEqual(response.status_code, 200)

        # Edit the comment
        response = self.client.post(reverse('exercisecomment-edit', kwargs={'pk': 1}),
                                    {'comment': comment_after})
        comments = exercise_1.exercisecomment_set.all()
        comment_1 = comments[0]

        if fail:
            self.assertEqual(comment_1.comment, comment_before)
        else:
            self.assertEqual(comment_1.comment, comment_after)

    def test_exercise_edit_comment_no_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login('test')
        self.exercise_edit_comment(fail=True)

    def test_exercise_edit_comment_not_logged_in(self):
        '''
        Tests the exercise comments
        '''

        self.exercise_edit_comment(fail=True)

    def test_exercise_edit_comment_authorized(self):
        '''
        Tests the exercise comments
        '''

        self.user_login()
        self.exercise_edit_comment(fail=False)



from django.conf.urls import patterns, url
from django.contrib.auth.decorators import permission_required

from wger.exercises.views import ExerciseUpdateView
from wger.exercises.views import ExerciseAddView
from wger.exercises.views import ExerciseDeleteView

from wger.exercises.views import ExerciseCategoryAddView
from wger.exercises.views import ExerciseCategoryUpdateView
from wger.exercises.views import ExerciseCategoryDeleteView

from wger.exercises.views import ExerciseCommentAddView
from wger.exercises.views import ExerciseCommentEditView

from wger.exercises.views import MuscleListView

urlpatterns = patterns('wger.exercises.views',

    # Exercises
    url(r'^overview/$', 'exercise_overview'),

    url(r'^muscle/overview/$',
        MuscleListView.as_view(),
        name='muscle-overview'),
    url(r'^search/$', 'exercise_search'),
    url(r'^(?P<id>\d+)/view/(?P<slug>[-\w]+)/$', 'exercise_view'),
    url(r'^(?P<id>\d+)/view/$', 'exercise_view'),
    url(r'^add/$',
        permission_required('exercises.change_exercise')(ExerciseAddView.as_view()),
        name='exercise-add'),
    url(r'^(?P<pk>\d+)/edit/$',
        permission_required('exercises.change_exercise')(ExerciseUpdateView.as_view()),
        name='exercise-edit'),
    url(r'^(?P<pk>\d+)/delete/$',
        permission_required('exercises.change_exercise')(ExerciseDeleteView.as_view()),
        name='exercise-delete'),

    # Comments
    url(r'^(?P<exercise_pk>\d+)/comment/add/$',
        permission_required('exercises.change_exercise')(ExerciseCommentAddView.as_view()),
        name='exercisecomment-add'),
    url(r'^comment/(?P<pk>\d+)/edit/$',
        permission_required('exercises.change_exercise')(ExerciseCommentEditView.as_view()),
        name='exercisecomment-edit'),
    url(r'^comment/(?P<id>\d+)/delete/$',
        'exercisecomment_delete',
        name='exercisecomment-delete'),

    # Categories
    url(r'^category/(?P<pk>\d+)/edit/$',
        permission_required('exercises.change_exercise')(ExerciseCategoryUpdateView.as_view()),
        name='exercisecategory-edit'),
    url(r'^category/add/$',
        permission_required('exercises.change_exercise')(ExerciseCategoryAddView.as_view()),
        name='exercisecategory-add'),
    url(r'^category/(?P<pk>\d+)/delete/$',
        permission_required('exercises.change_exercise')(ExerciseCategoryDeleteView.as_view()),
        name='exercisecategory-delete'),
)



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
import logging
import json
import uuid

from django.template import RequestContext
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.forms import ModelForm
from django.forms import ModelChoiceField
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.contrib.auth.decorators import permission_required
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy

from django.views.generic import ListView
from django.views.generic import DeleteView
from django.views.generic import CreateView
from django.views.generic import UpdateView

from wger.manager.utils import load_language

from wger.manager.models import WorkoutLog

from wger.exercises.models import Exercise
from wger.exercises.models import ExerciseComment
from wger.exercises.models import ExerciseCategory
from wger.exercises.models import Muscle

from wger.workout_manager.generic_views import YamlFormMixin
from wger.workout_manager.generic_views import YamlDeleteMixin


logger = logging.getLogger('workout_manager.custom')


# ************************
#    Exercise comments
# ************************
class ExerciseCommentForm(ModelForm):
    class Meta:
        model = ExerciseComment
        exclude = ('exercise',)


class ExerciseCommentEditView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing exercise comment
    '''

    model = ExerciseComment
    form_class = ExerciseCommentForm
    title = ugettext_lazy('Edit exercise comment')

    def get_success_url(self):
        return reverse('wger.exercises.views.exercise_view', kwargs={'id': self.object.exercise.id})

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(ExerciseCommentEditView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('exercisecomment-edit',
                                         kwargs={'pk': self.object.id})

        return context


class ExerciseCommentAddView(YamlFormMixin, CreateView):
    '''
    Generic view to add a new exercise comment
    '''

    model = ExerciseComment
    form_class = ExerciseCommentForm
    title = ugettext_lazy('Add exercise comment')

    def form_valid(self, form):
        form.instance.exercise = Exercise.objects.get(pk=self.kwargs['exercise_pk'])

        return super(ExerciseCommentAddView, self).form_valid(form)

    def get_success_url(self):
        return reverse('wger.exercises.views.exercise_view', kwargs={'id': self.object.exercise.id})

    def get_context_data(self, **kwargs):
        '''
        Send some additional data to the template
        '''
        context = super(ExerciseCommentAddView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('exercisecomment-add',
                                         kwargs={'exercise_pk': self.kwargs['exercise_pk']})

        return context


@permission_required('manager.add_exercisecomment')
def exercisecomment_delete(request, id):
    # Load the comment
    comment = get_object_or_404(ExerciseComment, pk=id)
    exercise_id = comment.exercise.id
    comment.delete()

    return HttpResponseRedirect(reverse('wger.exercises.views.exercise_view',
                                kwargs={'id': exercise_id}))


# ************************
#         Exercises
# ************************
def exercise_overview(request):
    '''
    Overview with all exercises
    '''
    language = load_language()

    template_data = {}
    template_data.update(csrf(request))

    template_data['categories'] = ExerciseCategory.objects.filter(language=language.id)

    return render_to_response('overview.html',
                              template_data,
                              context_instance=RequestContext(request))


class MuscleListView(ListView):
    '''
    Overview of all muscles and their exercises
    '''
    model = Muscle
    queryset = Muscle.objects.all().order_by('-is_front', 'name'),
    context_object_name = 'muscle_list'
    template_name = 'muscle_overview.html'


def exercise_view(request, id, slug=None):
    '''
    Detail view for an exercise
    '''

    template_data = {}
    template_data['comment_edit'] = False

    # Load the exercise itself
    exercise = get_object_or_404(Exercise, pk=id)
    template_data['exercise'] = exercise

    # Create the backgrounds that show what muscles the exercise works on
    backgrounds_back = []
    backgrounds_front = []

    for muscle in exercise.muscles.all():
        if muscle.is_front:
            backgrounds_front.append('images/muscles/main/muscle-%s.svg' % muscle.id)
        else:
            backgrounds_back.append('images/muscles/main/muscle-%s.svg' % muscle.id)

    for muscle in exercise.muscles_secondary.all():
        if muscle.is_front:
            backgrounds_front.append('images/muscles/secondary/muscle-%s.svg' % muscle.id)
        else:
            backgrounds_back.append('images/muscles/secondary/muscle-%s.svg' % muscle.id)

    # Append the "main" background, with the silhouette of the human body
    # This has to happen as the last step, so it is rendered behind the muscles.
    if backgrounds_front:
        backgrounds_front.append('images/muscles/muscular_system_front.svg')

    if backgrounds_back:
        backgrounds_back.append('images/muscles/muscular_system_back.svg')

    template_data['muscle_backgrounds_front'] = backgrounds_front
    template_data['muscle_backgrounds_back'] = backgrounds_back

    # If the user is logged in, load the log and prepare the entries for
    # rendering in the D3 chart
    entry = WorkoutLog()
    entry_log = []
    chart_data = []
    if request.user.is_authenticated():
        logs = WorkoutLog.objects.filter(user=request.user,
                                         exercise=exercise)

        entry_log, chart_data = entry.process_log_entries(logs)

    template_data['logs'] = entry_log
    template_data['json'] = chart_data
    template_data['svg_uuid'] = str(uuid.uuid4())
    template_data['reps'] = _("Reps")

    # Render
    return render_to_response('view.html',
                              template_data,
                              context_instance=RequestContext(request))


class ExercisesEditAddView(YamlFormMixin):
    '''
    Generic view to subclass from for exercise adding and editing, since they
    share all this settings
    '''
    model = Exercise

    form_fields = ['name',
                   'category',
                   'muscles',
                   'muscles_secondary',
                   'description']

    title = ugettext_lazy('Add exercise')
    custom_js = 'init_tinymce();'
    clean_html = ('description', )

    def get_form_class(self):

        # Define the exercise form here because only at this point during the request
        # have we access to the currently used language. In other places Django defaults
        # to 'en-us'.
        class ExerciseForm(ModelForm):
            language = load_language()
            category = ModelChoiceField(queryset=ExerciseCategory.objects.filter(
                                        language=language.id)
                                        )

            class Meta:
                model = Exercise

            class Media:
                js = ('js/tinymce/tiny_mce.js',)

        return ExerciseForm


class ExerciseUpdateView(ExercisesEditAddView, UpdateView):
    '''
    Generic view to update an existing exercise
    '''

    def get_context_data(self, **kwargs):
        context = super(ExerciseUpdateView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('exercise-edit', kwargs={'pk': self.object.id})
        context['title'] = _('Edit %s') % self.object.name

        return context


class ExerciseAddView(ExercisesEditAddView, CreateView):
    '''
    Generic view to add a new exercise
    '''

    form_action = reverse_lazy('exercise-add')


class ExerciseDeleteView(YamlDeleteMixin, DeleteView):
    '''
    Generic view to delete an existing exercise
    '''

    model = Exercise
    success_url = reverse_lazy('wger.exercises.views.exercise_overview')
    delete_message = ugettext_lazy('This will delete the exercise from all workouts.')

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(ExerciseDeleteView, self).get_context_data(**kwargs)

        context['title'] = _('Delete exercise %s?') % self.object.name
        context['form_action'] = reverse('exercise-delete', kwargs={'pk': self.kwargs['pk']})

        return context


def exercise_search(request):
    '''
    Search an exercise, return the result as a JSON list
    '''

    # Perform the search
    q = request.GET.get('term', '')
    user_language = load_language()
    exercises = Exercise.objects.filter(name__icontains=q, category__language_id=user_language)\
                                .order_by('category__name', 'name')

    # AJAX-request, this comes from the autocompleter. Create a list and send
    # it back as JSON
    if request.is_ajax():

        results = []
        for exercise in exercises:
            exercise_json = {}
            exercise_json['id'] = exercise.id
            exercise_json['name'] = exercise.name
            exercise_json['value'] = exercise.name
            exercise_json['category'] = exercise.category.name

            results.append(exercise_json)
        data = json.dumps(results)

        # Return the results to the server
        mimetype = 'application/json'
        return HttpResponse(data, mimetype)

    # Usual search (perhaps JS disabled), present the results as normal HTML page
    else:
        template_data = {}
        template_data.update(csrf(request))
        template_data['exercises'] = exercises
        template_data['search_term'] = q
        return render_to_response('exercise_search.html',
                                  template_data,
                                  context_instance=RequestContext(request))


# ************************
#   Exercise categories
# ************************
class ExerciseCategoryForm(ModelForm):
    class Meta:
        model = ExerciseCategory
        exclude = ('language',)


class ExerciseCategoryAddView(YamlFormMixin, CreateView):
    '''
    Generic view to add a new exercise category
    '''

    model = ExerciseCategory
    form_class = ExerciseCategoryForm
    success_url = reverse_lazy('wger.exercises.views.exercise_overview')
    title = ugettext_lazy('Add category')
    form_action = reverse_lazy('exercisecategory-add')

    def form_valid(self, form):
        form.instance.language = load_language()

        return super(ExerciseCategoryAddView, self).form_valid(form)


class ExerciseCategoryUpdateView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing exercise category
    '''

    model = ExerciseCategory
    form_class = ExerciseCategoryForm
    success_url = reverse_lazy('wger.exercises.views.exercise_overview')

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(ExerciseCategoryUpdateView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('exercisecategory-edit', kwargs={'pk': self.object.id})
        context['title'] = _('Edit %s') % self.object.name

        return context

    def form_valid(self, form):
        form.instance.language = load_language()

        return super(ExerciseCategoryUpdateView, self).form_valid(form)


class ExerciseCategoryDeleteView(YamlDeleteMixin, DeleteView):
    '''
    Generic view to delete an existing exercise category
    '''

    model = ExerciseCategory
    success_url = reverse_lazy('wger.exercises.views.exercise_overview')
    delete_message = ugettext_lazy('This will also delete all exercises in this category.')

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(ExerciseCategoryDeleteView, self).get_context_data(**kwargs)

        context['title'] = _('Delete category %s?') % self.object.name
        context['form_action'] = reverse('exercise-delete', kwargs={'pk': self.kwargs['pk']})

        return context



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Workout Manager.  If not, see <http://www.gnu.org/licenses/>.

from django.core.urlresolvers import reverse

from wger.nutrition.models import Ingredient

from wger.workout_manager.constants import NUTRITION_TAB

from wger.manager.tests.testcase import WorkoutManagerTestCase
from wger.manager.tests.testcase import WorkoutManagerDeleteTestCase
from wger.manager.tests.testcase import WorkoutManagerEditTestCase
from wger.manager.tests.testcase import WorkoutManagerAddTestCase


class DeleteIngredientTestCase(WorkoutManagerDeleteTestCase):
    '''
    Tests deleting an ingredient
    '''

    object_class = Ingredient
    url = 'ingredient-delete'
    pk = 1


class EditIngredientTestCase(WorkoutManagerEditTestCase):
    '''
    Tests editing an ingredient
    '''

    object_class = Ingredient
    url = 'ingredient-edit'
    pk = 1
    data = {'name': 'A new name',
            'sodium': 2,
            'energy': 200,
            'fat': 10,
            'carbohydrates_sugar': 5,
            'fat_saturated': 3.1415,
            'fibres': 2.1,
            'protein': 30,
            'carbohydrates': 10}


class AddIngredientTestCase(WorkoutManagerAddTestCase):
    '''
    Tests adding an ingredient
    '''

    object_class = Ingredient
    url = 'ingredient-add'
    pk = 7
    data = {'name': 'A new ingredient',
            'sodium': 2,
            'energy': 200,
            'fat': 10,
            'carbohydrates_sugar': 5,
            'fat_saturated': 3.1415,
            'fibres': 2.1,
            'protein': 30,
            'carbohydrates': 10}


class IngredientDetailTestCase(WorkoutManagerTestCase):
    '''
    Tests the ingredient details page
    '''

    def ingredient_detail(self, editor=False):
        '''
        Tests the ingredient details page
        '''

        response = self.client.get(reverse('wger.nutrition.views.ingredient_view',
                                   kwargs={'id': 6}))
        self.assertEqual(response.status_code, 200)

        # Correct tab is selected
        self.assertEqual(response.context['active_tab'], NUTRITION_TAB)
        self.assertTrue(response.context['ingredient'])

        # Only authorized users see the edit links
        if editor:
            self.assertContains(response, 'Edit ingredient')
            self.assertContains(response, 'Delete ingredient')
        else:
            self.assertNotContains(response, 'Edit ingredient')
            self.assertNotContains(response, 'Delete ingredient')

        # Non-existent ingredients throw a 404.
        response = self.client.get(reverse('wger.nutrition.views.ingredient_view',
                                   kwargs={'id': 42}))
        self.assertEqual(response.status_code, 404)

    def test_ingredient_detail_editor(self):
        '''
        Tests the ingredient details page as a logged in user with editor rights
        '''

        self.user_login('admin')
        self.ingredient_detail(editor=True)

    def test_ingredient_detail_non_editor(self):
        '''
        Tests the ingredient details page as a logged in user without editor rights
        '''

        self.user_login('test')
        self.ingredient_detail(editor=False)

    def test_ingredient_detail_logged_out(self):
        '''
        Tests the ingredient details page as an anonymous (logged out) user
        '''

        self.ingredient_detail(editor=False)



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

from django.core.urlresolvers import reverse

from wger.manager.models import TrainingSchedule

from wger.manager.tests.testcase import WorkoutManagerTestCase


class CopyWorkoutTestCase(WorkoutManagerTestCase):
    '''
    Tests copying a workout
    '''

    def copy_workout(self, logged_in=False):
        '''
        Helper function to test copying workouts
        '''

        # Open the copy workout form
        response = self.client.get(reverse('workout-copy', kwargs={'pk': '3'}))
        if not logged_in:
            self.assertEqual(response.status_code, 302)
        else:
            self.assertEqual(response.status_code, 200)

        # Copy the workout
        count_before = TrainingSchedule.objects.count()
        response = self.client.post(reverse('workout-copy', kwargs={'pk': '3'}),
                                    {'comment': 'A copied workout'})
        count_after = TrainingSchedule.objects.count()

        if not logged_in:
            self.assertEqual(count_before, count_after)
        else:
            self.assertGreater(count_after, count_before)
            self.assertEqual(count_after, 4)

            self.assertTemplateUsed('workout/view.html')

        # Test accessing the copied workout
        response = self.client.get(reverse('wger.manager.views.view_workout', kwargs={'id': 4}))

        if not logged_in:
            self.assertEqual(response.status_code, 302)
        else:
            self.assertEqual(response.status_code, 200)

    def test_copy_workout_anonymous(self):
        '''
        Test copying a workout as anonymous user
        '''

        self.copy_workout()

    def test_copy_workout_logged_in(self):
        '''
        Test copying a workout as a logged in user
        '''

        self.user_login('test')
        self.copy_workout(logged_in=True)



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
import logging
import json

from django.template import RequestContext
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.http import HttpResponseForbidden
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.contrib.auth.decorators import login_required
from django.forms import ModelForm
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy


from django.views.generic import DeleteView
from django.views.generic import CreateView
from django.views.generic import UpdateView


from wger.nutrition.models import NutritionPlan
from wger.nutrition.models import Meal
from wger.nutrition.models import MealItem
from wger.nutrition.models import Ingredient

from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.pagesizes import A4, cm
from reportlab.platypus import Paragraph, SimpleDocTemplate, Table
from reportlab.lib import colors

from wger.manager.utils import load_language
from wger.manager.utils import load_ingredient_languages

from wger.workout_manager import get_version
from wger.workout_manager.generic_views import YamlFormMixin
from wger.workout_manager.generic_views import YamlDeleteMixin
from wger.workout_manager import helpers


logger = logging.getLogger('workout_manager.custom')


# ************************
# Plan functions
# ************************

class PlanForm(ModelForm):
    class Meta:
        model = NutritionPlan
        exclude = ('user', 'language', )


@login_required
def overview(request):
    template_data = {}
    template_data.update(csrf(request))

    plans = NutritionPlan.objects.filter(user=request.user)
    template_data['plans'] = plans

    return render_to_response('nutrition_overview.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def add(request):
    '''
    Add a new nutrition plan and redirect to its page
    '''

    plan = NutritionPlan()
    plan.user = request.user
    plan.language = load_language()
    plan.save()

    return HttpResponseRedirect(reverse('wger.nutrition.views.view', kwargs={'id': plan.id}))


class PlanDeleteView(YamlDeleteMixin, DeleteView):
    '''
    Generic view to delete a nutritional plan
    '''

    model = NutritionPlan
    success_url = reverse_lazy('wger.nutrition.views.overview')
    title = ugettext_lazy('Delete nutritional plan?')
    form_action_urlname = 'nutrition-delete'


class PlanEditView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing nutritional plan
    '''

    model = NutritionPlan
    form_class = PlanForm
    title = ugettext_lazy('Add a new nutritional plan')
    form_action_urlname = 'nutrition-edit'


@login_required
def view(request, id):
    '''
    Show the nutrition plan with the given ID
    '''
    template_data = {}

    plan = get_object_or_404(NutritionPlan, pk=id, user=request.user)
    template_data['plan'] = plan

    # Load the language and pass it to the template
    language = load_language()
    template_data['language'] = language

    # Get the nutrional info

    template_data['nutritional_data'] = plan.get_nutritional_values()

    return render_to_response('view_nutrition_plan.html',
                              template_data,
                              context_instance=RequestContext(request))


@login_required
def copy(request, pk):
    '''
    Copy the nutrition plan
    '''

    plan = get_object_or_404(NutritionPlan, pk=pk, user=request.user)

    # Copy plan
    meals = plan.meal_set.all()

    plan_copy = plan
    plan_copy.pk = None
    plan_copy.save()

    # Copy the meals
    for meal in meals:
        meal_items = meal.mealitem_set.all()

        meal_copy = meal
        meal_copy.pk = None
        meal_copy.plan = plan_copy
        meal_copy.save()

        # Copy the individual meal entries
        for item in meal_items:
            item_copy = item
            item_copy.pk = None
            item_copy.meal = meal_copy
            item.save()

    # Redirect
    return HttpResponseRedirect(reverse('wger.nutrition.views.view', kwargs={'id': plan.id}))


@login_required
def export_pdf(request, id):
    '''
    Generates a PDF with the contents of a nutrition plan

    See also
    * http://www.blog.pythonlibrary.org/2010/09/21/reportlab
    * http://www.reportlab.com/apis/reportlab/dev/platypus.html
    '''

    #Load the workout
    plan = get_object_or_404(NutritionPlan, pk=id, user=request.user)

    # Create the HttpResponse object with the appropriate PDF headers.
    response = HttpResponse(mimetype='application/pdf')

    # Translators: translation can only have ASCII characters
    response['Content-Disposition'] = 'attachment; filename=%s.pdf' % _('nutritional-plan')

    # Create the PDF object, using the response object as its "file."
    doc = SimpleDocTemplate(response,
                            pagesize=A4,
                            title=_('Workout'),
                            author=_('Workout Manager'),
                            subject=_('Nutritional plan %s') % request.user.username)

    # Background colour for header
    # Reportlab doesn't use the HTML hexadecimal format, but has a range of
    # 0 till 1, so we have to convert here.
    header_colour = colors.Color(int('73', 16) / 255.0,
                                 int('8a', 16) / 255.0,
                                 int('5f', 16) / 255.0)

    # container for the 'Flowable' objects
    elements = []

    styleSheet = getSampleStyleSheet()
    data = []

    # Iterate through the Plan
    meal_markers = []
    ingredient_markers = []

    # Meals
    i = 0
    for meal in plan.meal_set.select_related():
        i += 1

        meal_markers.append(len(data))

        if not meal.time:
            P = Paragraph('<para align="center"><strong>%(meal_nr)s</strong></para>' %
                          {'meal_nr': i},
                          styleSheet["Normal"])
        else:
            P = Paragraph('<para align="center"><strong>%(meal_nr)s - '
                          '%(meal_time)s</strong></para>' %
                          {'meal_nr': i,
                          'meal_time': meal.time.strftime("%H:%M")},
                          styleSheet["Normal"])
        data.append([P])

        # Ingredients
        for item in meal.mealitem_set.select_related():
            ingredient_markers.append(len(data))

            P = Paragraph('<para>%s</para>' % item.ingredient.name,
                          styleSheet["Normal"])
            data.append(["%sg" % item.amount_gramm, P])

    # Set general table styles

    #('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),
    #('BOX', (0,0), (-1,-1), 0.25, colors.black)
    table_style = []

    # Set specific styles, e.g. background for title cells
    for marker in meal_markers:
        # Set background colour for headings
        table_style.append(('BACKGROUND', (0, marker), (-1, marker), header_colour))
        table_style.append(('BOX', (0, marker), (-1, marker), 1.25, colors.black))

        # Make the headings span the whole width
        table_style.append(('SPAN', (0, marker), (-1, marker)))

    t = Table(data, style=table_style)

    # Manually set the width of the columns
    t._argW[0] = 2 * cm

    # Set the title (if available)
    if plan.description:
        P = Paragraph('<para align="center"><strong>%(description)s</strong></para>' %
                      {'description': plan.description},
                      styleSheet["Normal"])
        elements.append(P)

        # Filler
        P = Paragraph('<para> </para>', styleSheet["Normal"])
        elements.append(P)

    # append the table to the document
    elements.append(t)

    # Footer, add filler paragraph
    P = Paragraph('<para> </para>', styleSheet["Normal"])
    elements.append(P)

    # Print date and info
    P = Paragraph('<para align="left">%(date)s - %(created)s v%(version)s</para>' %
                  {'date': _("Created on the <b>%s</b>") %
                  plan.creation_date.strftime("%d.%m.%Y"),
                  'created': "Workout Manager",
                  'version': get_version()},
                  styleSheet["Normal"])
    elements.append(P)

    doc.build(elements)

    return response


# ************************
# Meal functions
# ************************

class MealForm(ModelForm):
    class Meta:
        model = Meal
        exclude = ('plan', 'order')


class MealItemForm(ModelForm):
    class Meta:
        model = MealItem
        exclude = ('meal', 'order')


class MealCreateView(YamlFormMixin, CreateView):
    '''
    Generic view to add a new meal to a nutrition plan
    '''

    model = Meal
    form_class = MealForm
    title = ugettext_lazy('Add new meal')
    owner_object = {'pk': 'plan_pk', 'class': NutritionPlan}

    def form_valid(self, form):
        plan = get_object_or_404(NutritionPlan, pk=self.kwargs['plan_pk'], user=self.request.user)
        form.instance.plan = plan
        form.instance.order = 1
        return super(MealCreateView, self).form_valid(form)

    def get_success_url(self):
        return reverse('wger.nutrition.views.view', kwargs={'id': self.object.plan.id})

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(MealCreateView, self).get_context_data(**kwargs)
        context['form_action'] = reverse('meal-add',
                                         kwargs={'plan_pk': self.kwargs['plan_pk']})

        return context


class MealEditView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing meal
    '''

    model = Meal
    form_class = MealForm
    title = ugettext_lazy('Edit meal')
    form_action_urlname = 'meal-edit'

    def get_success_url(self):
        return reverse('wger.nutrition.views.view', kwargs={'id': self.object.plan.id})


@login_required
def delete_meal(request, id):
    '''
    Deletes the meal with the given ID
    '''

    # Load the meal
    meal = get_object_or_404(Meal, pk=id)
    plan = meal.plan

    # Only delete if the user is the owner
    if plan.user == request.user:
        meal.delete()
        return HttpResponseRedirect(reverse('wger.nutrition.views.view', kwargs={'id': plan.id}))
    else:
        return HttpResponseForbidden()


# ************************
# Meal ingredient functions
# ************************
@login_required
def delete_meal_item(request, item_id):
    '''
    Deletes the meal ingredient with the given ID
    '''

    # Load the item
    item = get_object_or_404(MealItem, pk=item_id)
    plan = item.meal.plan

    # Only delete if the user is the owner
    if plan.user == request.user:
        item.delete()
        return HttpResponseRedirect(reverse('wger.nutrition.views.view', kwargs={'id': plan.id}))
    else:
        return HttpResponseForbidden()


@login_required
def edit_meal_item(request, id, meal_id, item_id=None):
    '''
    Form to add a meal to a plan
    '''
    template_data = {}

    # Load the plan
    plan = get_object_or_404(NutritionPlan, pk=id, user=request.user)
    template_data['plan'] = plan

    # Load the meal
    meal = get_object_or_404(Meal, pk=meal_id)
    template_data['meal'] = meal

    if meal.plan != plan:
        return HttpResponseForbidden()

    # Load the meal item
    # If the object is new, we will receice a 'None' (string) as the ID
    # from the template, so we check for it (ValueError) and for an actual
    # None (TypeError)
    try:
        int(item_id)
        meal_item = get_object_or_404(MealItem, pk=item_id)
        template_data['ingredient'] = meal_item.ingredient.id
        template_data['ingredient_searchfield'] = meal_item.ingredient.name

    except ValueError, TypeError:
        meal_item = MealItem()

    template_data['meal_item'] = meal_item

    # Check that javascript is activated This is used to hide the drop down list with the
    # ingredients and use the autocompleter version instead. This is not only nicer but also much
    # faster as the ingredient list from USDA is many thosand entries long.
    js_active = request.GET.get('js', False)
    template_data['js_active'] = js_active

    # Process request
    if request.method == 'POST':
        meal_form = MealItemForm(request.POST, instance=meal_item)

        # Pass the ingredient ID back to the template, this is originally set by the JQuery
        # autocompleter, in case of errors (max. gramms reached), we need to set the ID manually
        # so the user can simply submit again.
        template_data['ingredient'] = request.POST.get('ingredient', '')

        template_data['ingredient_searchfield'] = request.POST.get('ingredient_searchfield', '')

        # If the data is valid, save and redirect
        if meal_form.is_valid():
            meal_item = meal_form.save(commit=False)
            meal_item.meal = meal
            meal_item.order = 1
            meal_item.save()

            return HttpResponseRedirect(reverse('wger.nutrition.views.view', kwargs={'id': id}))

    else:
        meal_form = MealItemForm(instance=meal_item)
    template_data['form'] = meal_form

    return render_to_response('edit_meal_item.html',
                              template_data,
                              context_instance=RequestContext(request))


# ************************
# Ingredient functions
# ************************
def ingredient_overview(request):
    '''
    Show an overview of all ingredients
    '''

    template_data = {}

    # Filter the ingredients the user will see by its language
    # (the user can also want to see ingredients in English, see load_ingredient_languages)
    languages = load_ingredient_languages(request)

    # Load the ingredients and paginate it
    ingredients_list = Ingredient.objects.filter(language__in=languages)

    pagination = helpers.pagination(ingredients_list, request.GET.get('page'))
    template_data['page_range'] = pagination['page_range']
    template_data['ingredients'] = pagination['page']

    return render_to_response('ingredient_overview.html',
                              template_data,
                              context_instance=RequestContext(request))


def ingredient_view(request, id, slug=None):
    template_data = {}

    ingredient = get_object_or_404(Ingredient, pk=id)
    template_data['ingredient'] = ingredient

    return render_to_response('view_ingredient.html',
                              template_data,
                              context_instance=RequestContext(request))


class IngredientForm(ModelForm):
    class Meta:
        model = Ingredient
        exclude = ('language',)


class IngredientDeleteView(YamlDeleteMixin, DeleteView):
    '''
    Generic view to delete an existing ingredient
    '''

    model = Ingredient
    template_name = 'delete.html'
    success_url = reverse_lazy('wger.nutrition.views.ingredient_overview')

    # Send some additional data to the template
    def get_context_data(self, **kwargs):
        context = super(IngredientDeleteView, self).get_context_data(**kwargs)

        context['title'] = _('Delete %s?') % self.object.name
        context['form_action'] = reverse('ingredient-delete', kwargs={'pk': self.kwargs['pk']})

        return context


class IngredientEditView(YamlFormMixin, UpdateView):
    '''
    Generic view to update an existing ingredient
    '''

    model = Ingredient
    form_class = IngredientForm
    title = ugettext_lazy('Add a new ingredient')
    form_action_urlname = 'ingredient-edit'


class IngredientCreateView(YamlFormMixin, CreateView):
    '''
    Generic view to add a new ingredient
    '''

    model = Ingredient
    form_class = IngredientForm
    title = ugettext_lazy('Add a new ingredient')
    form_action = reverse_lazy('ingredient-add')

    def form_valid(self, form):
        form.instance.language = load_language()
        return super(IngredientCreateView, self).form_valid(form)


def ingredient_search(request):
    '''
    Search for an exercise, return the result as a JSON list or as HTML page, depending on how
    the function was invoked
    '''

    # Filter the ingredients the user will see by its language
    # (the user can also want to see ingredients in English, see load_ingredient_languages)
    languages = load_ingredient_languages(request)

    # Perform the search
    q = request.GET.get('term', '')
    ingredients = Ingredient.objects.filter(name__icontains=q, language__in=languages)

    # AJAX-request, this comes from the autocompleter. Create a list and send it back as JSON
    if request.is_ajax():

        results = []
        for ingredient in ingredients:
            ingredient_json = {}
            ingredient_json['id'] = ingredient.id
            ingredient_json['name'] = ingredient.name
            ingredient_json['value'] = ingredient.name
            results.append(ingredient_json)
        data = json.dumps(results)

        # Return the results to the server
        mimetype = 'application/json'
        return HttpResponse(data, mimetype)

    # Usual search (perhaps JS disabled), present the results as normal HTML page
    else:
        template_data = {}
        template_data.update(csrf(request))
        template_data['ingredients'] = ingredients
        template_data['search_term'] = q
        return render_to_response('ingredient_search.html',
                                  template_data,
                                  context_instance=RequestContext(request))



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

import datetime

from django.core.urlresolvers import reverse

from wger.weight.models import WeightEntry

from wger.manager.tests.testcase import WorkoutManagerTestCase


class AddWeightEntryTestCase(WorkoutManagerTestCase):
    '''
    Tests adding a weight entry
    '''

    def add_entry(self, fail=False):
        '''
        Helper function to test adding a weight entry
        '''

        # Fetch the add entry page
        response = self.client.get(reverse('weight-add'))

        if fail:
            self.assertIn(response.status_code, (302, 403))
            self.assertTemplateUsed('login.html')

        else:
            self.assertEqual(response.status_code, 200)

        # Enter the data
        response = self.client.post(reverse('weight-add'),
                                    {'weight': '81',
                                    'creation_date': '2013-02-01'})
        if fail:
            self.assertRaises(WeightEntry.DoesNotExist, WeightEntry.objects.get, pk=8)
            self.assertIn(response.status_code, (302, 403))
            self.assertTemplateUsed('login.html')

        else:
            entry = WeightEntry.objects.get(pk=8)
            self.assertEqual(response.status_code, 302)
            self.assertEqual(entry.weight, 81)
            self.assertEqual(entry.creation_date, datetime.date(2013, 2, 1))

    def test_add_entry_anonymous(self):
        '''
        Test adding a weight entry as an anonymous user
        '''

        self.add_entry(fail=True)

    def test_add_entry_logged_in(self):
        '''
        Test adding a weight entry as a logged in user
        '''

        self.user_login('test')
        self.add_entry(fail=False)


class EditWeightEntryTestCase(WorkoutManagerTestCase):
    '''
    Tests editing a weight entry
    '''

    def edit_entry(self, fail=False):
        '''
        Helper function
        '''

        # Fetch the day edit page
        response = self.client.get(reverse('weight-edit', kwargs={'pk': 1}))

        if fail:
            self.assertIn(response.status_code, (302, 403))
            self.assertTemplateUsed('login.html')

        else:
            self.assertEqual(response.status_code, 200)

        # Edit the day
        response = self.client.post(reverse('weight-edit', kwargs={'pk': 1}),
                                    {'weight': 100,
                                    'creation_date': '2005-01-01'})

        entry = WeightEntry.objects.get(pk=1)

        if fail:
            self.assertIn(response.status_code, (302, 403))
            self.assertTemplateUsed('login.html')
            self.assertEqual(entry.weight, 77)
            self.assertEqual(entry.creation_date, datetime.date(2012, 10, 1))

        else:
            self.assertEqual(response.status_code, 302)
            self.assertEqual(entry.weight, 100)
            self.assertEqual(entry.creation_date, datetime.date(2005, 1, 1))

    def test_edit_entry_anonymous(self):
        '''
        Test editing a weight entry as an anonymous user
        '''

        self.edit_entry(fail=True)

    def test_edit_entry_owner(self):
        '''
        Test editing a weight entry as the owner user
        '''

        self.user_login('test')
        self.edit_entry(fail=False)

    def test_edit_entry_other(self):
        '''
        Test editing a weight entry as a different user
        '''

        self.user_login('admin')
        self.edit_entry(fail=True)



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

import decimal
import json

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from wger.workout_manager.constants import PAGINATION_OBJECTS_PER_PAGE
from wger.workout_manager.constants import PAGINATION_MAX_TOTAL_PAGES
from wger.workout_manager.constants import PAGINATION_PAGES_AROUND_CURRENT


def pagination(object_list,
               request_page,
               paginator_class=Paginator,
               objects_per_page=PAGINATION_OBJECTS_PER_PAGE,
               max_total_pages=PAGINATION_MAX_TOTAL_PAGES,
               pages_around_current=PAGINATION_PAGES_AROUND_CURRENT):
    '''
    Helper function to initialise the pagination.

    If the list is too long, only pages around the current one are shown.
    '''

    paginator = paginator_class(object_list, objects_per_page)

    try:
        paginated_page = paginator.page(request_page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        paginated_page = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        paginated_page = paginator.page(paginator.num_pages)

    # For very long lists (e.g. the English ingredient with more than 8000 items)
    # we muck around here to remove the pages not inmediately 'around' the current
    # one, otherwise we end up with a which a useless block with 300 pages.
    if paginator.num_pages > max_total_pages:

        start_page = paginated_page.number - pages_around_current
        for i in range(paginated_page.number - pages_around_current, paginated_page.number + 1):
            if i > 0:
                start_page = i
                break

        end_page = paginated_page.number + pages_around_current
        for i in range(paginated_page.number, paginated_page.number + pages_around_current):
            if i > paginator.num_pages:
                end_page = i

                break

        page_range = range(start_page, end_page)
    else:
        page_range = paginator.page_range

    # OK, return
    return({'page': paginated_page,
            'page_range': page_range})


class DecimalJsonEncoder(json.JSONEncoder):
    '''
    Custom JSON encoder.

    This class is needed because we store some data as a decimal (e.g. the
    individual weight entries in the workout log) and they need to be
    processed, json.dumps() doesn't work on them
    '''
    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            return str(obj)
            #return "%.2f" % obj
        return json.JSONEncoder.default(self, obj)



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License


from django.core.urlresolvers import reverse

from wger.exercises.models import ExerciseCategory
from wger.manager.tests.testcase import WorkoutManagerTestCase
from wger.manager.tests.testcase import WorkoutManagerDeleteTestCase


class DeleteExerciseCategoryTestCase(WorkoutManagerDeleteTestCase):
    '''
    Exercise category delete test case
    '''

    object_class = ExerciseCategory
    url = 'exercisecategory-delete'
    pk = 4
    user_success = 'admin'
    user_fail = 'test'


class EditExerciseCategoryTestCase(WorkoutManagerTestCase):
    '''
    Exercise category test case
    '''

    def edit_category(self, fail=False):
        '''
        Helper function to test editing categories
        '''

        category = ExerciseCategory.objects.get(pk=3)
        old_name = category.name

        response = self.client.get(reverse('exercisecategory-edit', kwargs={'pk': 3}))

        # Did it work
        if fail:
            self.assertEqual(response.status_code, 302)
        else:
            self.assertEqual(response.status_code, 200)

        response = self.client.post(reverse('exercisecategory-edit',
                                            kwargs={'pk': 3}),
                                    {'name': 'A different name'})

        # There is a redirect
        self.assertEqual(response.status_code, 302)

        category = ExerciseCategory.objects.get(pk=3)
        new_name = category.name

        # Did it work
        if fail:
            self.assertEqual(old_name,
                             new_name,
                             'Category was edited by unauthorzed user')
        else:
            self.assertTrue(old_name != new_name,
                            'Category wasnt deleted by unauthorzed user')

        # No name
        if not fail:
            response = self.client.post(reverse('exercisecategory-edit',
                                                kwargs={'pk': 3}),
                                        {'name': ''})

            self.assertTrue(response.context['form'].errors['name'])

    def test_edit_category_unauthorized(self):
        '''
        Test editing a category by an unauthorized user
        '''

        self.user_login('test')
        self.edit_category(fail=True)

    def test_edit_category_anonymous(self):
        '''
        Test editing a category by an anonymous user
        '''

        self.user_logout()
        self.edit_category(fail=True)

    def test_edit_category_authorized(self):
        '''
        Test editing a category by an authorized user
        '''

        self.user_login()
        self.edit_category()


class AddExerciseCategoryTestCase(WorkoutManagerTestCase):
    '''
    Exercise category test case
    '''

    def add_category(self, fail=False):
        '''
        Helper function to test adding categories
        '''

        response = self.client.get(reverse('exercisecategory-add'))

        # Did it work?
        if fail:
            self.assertEqual(response.status_code, 302)
        else:
            self.assertEqual(response.status_code, 200)

        count_before = ExerciseCategory.objects.count()
        response = self.client.post(reverse('exercisecategory-add'),
                                    {'name': 'A new category'})
        count_after = ExerciseCategory.objects.count()

        # There is a redirect
        self.assertEqual(response.status_code, 302)

        # Did it work
        if fail:
            self.assertEqual(count_before, count_after)
        else:
            self.assertGreater(count_after, count_before)

    def test_add_category_unauthorized(self):
        '''
        Test adding a category by an unauthorized user
        '''

        self.user_login('test')
        self.add_category(fail=True)

    def test_add_category_anonymous(self):
        '''
        Test adding a category by an anonymous user
        '''

        self.add_category(fail=True)

    def test_add_category_authorized(self):
        '''
        Test adding a category by an authorized user
        '''

        self.user_login()
        self.add_category()



# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License

from django.core.urlresolvers import reverse

from django.contrib.auth.models import User

from wger.manager.tests.testcase import WorkoutManagerTestCase


class DemoUserTestCase(WorkoutManagerTestCase):
    '''
    Tests creating a demo user
    '''

    def test_demo_user(self):

        # Open the copy workout form
        response = self.client.get(reverse('demo-account'))
        self.assertEqual(response.status_code, 200)

        # Submit the form
        count_before = User.objects.count()
        response = self.client.post(reverse('demo-account'),
                                    {'recaptcha_response_field': 'PASSED'})
        count_after = User.objects.count()

        self.assertEqual(response.status_code, 302)
        response = self.client.get(reverse('dashboard'))

        # Check that the demo user was correctly created
        self.assertEqual(count_before + 1, count_after)
        self.assertTrue(User.objects.filter(userprofile__is_temporary=1).count(), 1)



from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse_lazy

from wger.manager.views import WorkoutEditView
from wger.manager.views import WorkoutDeleteView
from wger.manager.views import WorkoutLogDetailView
from wger.manager.views import WorkoutLogUpdateView

from wger.manager.views import DayEditView
from wger.manager.views import DayCreateView

from wger.workout_manager.constants import USER_TAB


urlpatterns = patterns('wger.manager.views',

    # The landing page
    url(r'^$', 'index', name='index'),

    # The dashboard
    url(r'^dashboard$', 'dashboard', name='dashboard'),


    # User
    url(r'^logout$', 'logout', name='logout'),
    url(r'^user/registration$', 'registration', name='registration'),
    url(r'^user/demo-account$',
        'create_demo_user',
        name='demo-account'),

    url(r'^user/preferences$', 'preferences', name='preferences'),

    # Workout
    url(r'^workout/overview$', 'overview'),
    url(r'^workout/add$', 'add'),
    url(r'^workout/(?P<pk>\d+)/copy/$',
        'copy_workout',
        name='workout-copy'),

    url(r'^workout/(?P<pk>\d+)/edit/$',
        login_required(WorkoutEditView.as_view()),
        name='workout-edit'),
    url(r'^workout/(?P<pk>\d+)/delete/$',
        login_required(WorkoutDeleteView.as_view()),
        name='workout-delete'),
    url(r'^workout/(?P<id>\d+)/view/$',
        'view_workout',
        name='workout-view'),
    url(r'^workout/(?P<pk>\d+)/log/$',
        login_required(WorkoutLogDetailView.as_view()),
        name='workout-log'),
    url(r'^workout/log/edit-entry/(?P<pk>\d+)$',
        login_required(WorkoutLogUpdateView.as_view()),
        name='workout-log-edit'),


    # Days
    url(r'^workout/day/(?P<pk>\d+)/edit/$',
        login_required(DayEditView.as_view()),
        name='day-edit'),
    url(r'^workout/(?P<workout_pk>\d+)/day/add/$',
        login_required(DayCreateView.as_view()),
        name='day-add'),
    url(r'^workout/(?P<id>\d+)/delete/day/(?P<day_id>\d+)$', 'delete_day'),
    url(r'^workout/day/view/(?P<id>\d+)$', 'view_day'),
    url(r'^workout/day/(?P<pk>\d+)/log/add/$',
        'workout_log_add',
        name='day-log'),

    # Sets and Settings
    url(r'^workout/(?P<id>\d+)/day/(?P<day_id>\d+)/edit/set/(?P<set_id>\w*)$', 'edit_set'),
    url(r'^workout/(?P<id>\d+)/day/(?P<day_id>\d+)/delete/set/(?P<set_id>\d+)$', 'delete_set'),
    url(r'^workout/(?P<id>\d+)/set/(?P<set_id>\d+)/exercise/(?P<exercise_id>\d+)/edit/setting/(?P<setting_id>\d*)$', 'edit_setting'),
    url(r'^workout/(?P<id>\d+)/set/(?P<set_id>\d+)/exercise/(?P<exercise_id>\d+)/delete/setting$', 'delete_setting'),

    # AJAX
    url(r'^workout/api/edit-set$', 'api_edit_set'),
    url(r'^workout/api/edit-settting$', 'api_edit_setting'),
    url(r'^workout/api/user-preferences$', 'api_user_preferences'),
)

# PDF stuff is in a different file
urlpatterns = urlpatterns + patterns('wger.manager.pdf',
     url(r'^workout/(?P<id>\d+)/pdf/$', 'workout_log'))

# Password reset is implemented by Django, no need to cook our own soup here
# (besides the templates)
urlpatterns = urlpatterns + patterns('',
    url(r'^login/$',
        'django.contrib.auth.views.login',
        {'template_name': 'user/login.html',
         'extra_context': {'active_tab': USER_TAB}},
        name='login'),

    url(r'^user/password/change$',
        'django.contrib.auth.views.password_change',
        {'template_name': 'user/change_password.html',
          'post_change_redirect': reverse_lazy('index'),
          'extra_context': {'active_tab': USER_TAB}},
        name='change-password'),

    url(r'^user/password/reset/$',
        'django.contrib.auth.views.password_reset',
        {'template_name': 'user/password_reset_form.html'},
        name='password_reset'),

    url(r'^user/password/reset/done/$',
        'django.contrib.auth.views.password_reset_done',
        {'template_name': 'user/password_reset_done.html'},
        name='password_reset_done'),

    url(r'^user/password/reset/check/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        'django.contrib.auth.views.password_reset_confirm',
        {'template_name': 'user/password_reset_confirm.html'},
        name='password_reset_confirm'),

    url(r'^user/password/reset/complete/$',
        'django.contrib.auth.views.password_reset_complete',
        {'template_name': 'user/password_reset_complete.html'},
        name='password_reset_complete'),
    )



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License


'''
This file contains global settings that don't need to be changed by the user
'''

from workout_manager.settings_global import *

# Use 'DEBUG = True' to get more details for server errors
# (Default for releases: 'False')
DEBUG = True
TEMPLATE_DEBUG = DEBUG


# Make this unique, and don't share it with anybody.
SECRET_KEY = ''

# Database settings
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.',  # Add 'postgresql_psycopg2','mysql', 'sqlite3' or 'oracle'
        'NAME': '',                       # Or path to database file if using sqlite3.
        'USER': '',                       # Not used with sqlite3.
        'PASSWORD': '',                   # Not used with sqlite3.
        'HOST': '',                       # Set to empty string for localhost. Not used with sqlite3
        'PORT': '',                       # Set to empty string for default. Not used with sqlite3.
    }
}

# Your reCaptcha keys
RECAPTCHA_PUBLIC_KEY = ''
RECAPTCHA_PRIVATE_KEY = ''

# The site's URL (e.g. http://www.my-gym-site.com or http://localhost:8000)
# This is needed for Mozilla's BrowserID to work
SITE_URL = 'http://localhost:8000'



from django import template

register = template.Library()


@register.filter(name='get_current_settings')
def get_current_settings(exercise, set_id):
    '''
    Does a filter on the sets

    We need to do this here because it's not possible to pass arguments to function in the template,
    and we are only interested on the settings that belong to the current set
    '''
    return exercise.setting_set.filter(set_id=set_id)


@register.inclusion_tag('tags/render_day.html')
def render_day(day):
    '''
    Renders a day as it will be displayed in the workout overview
    '''
    return {'day':     day,
            'workout': day.training}


@register.inclusion_tag('tags/pagination.html')
def pagination(page, page_range):
    '''
    Renders the necessary links to paginating a long list
    '''
    return {'page':       page,
            'page_range': page_range}


@register.inclusion_tag('tags/render_weight_log.html')
def render_weight_log(log, div_uuid):
    '''
    Renders a weight log series
    '''

    return {'log': log,
            'div_uuid': div_uuid}


@register.inclusion_tag('tags/yaml_form_element.html')
def yaml_form_field(field, css_class='ym-fbox-text'):
    '''
    Renders a form field in a <tr> with all necessary CSS
    '''

    return {'field':     field,
            'css_class': css_class}


@register.inclusion_tag('tags/cc-by-sa-sidebar.html')
def cc_by_sa_sidebar(language):
    '''
    Renders a form field in a <tr> with all necessary CSS
    '''

    return {'language': language}



# -*- coding: utf-8 -*-

# This file is part of Workout Manager.
#
# Workout Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workout Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Workout Manager.  If not, see <http://www.gnu.org/licenses/>.


from django.db import models

from django.template.defaultfilters import slugify  # django.utils.text.slugify in django 1.5!
from django.core.validators import MaxValueValidator
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from wger.exercises.models import Language


class NutritionPlan(models.Model):
    '''
    A nutrition plan
    '''

    # Metaclass to set some other properties
    class Meta:

        # Order by creation_date, descending (oldest first)
        ordering = ["-creation_date", ]

    user = models.ForeignKey(User, verbose_name=_('User'))
    language = models.ForeignKey(Language, verbose_name=_('Language'))
    creation_date = models.DateField(_('Creation date'), auto_now_add=True)
    description = models.TextField(max_length=2000,
                                   blank=True,
                                   verbose_name=_('Description'),
                                   help_text=_('A description of the goal of the plan, e.g. '
                                   '"Gain mass" or "Prepare for summer"'))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "Nutrition plan for %s, %s" % (self.user, self.creation_date)

    def get_absolute_url(self):
        '''
        Returns the canonical URL to view this object
        '''
        return reverse('wger.nutrition.views.view', kwargs={'id': self.id})

    def get_nutritional_values(self):
        # Sum the nutrional info
        nutritional_info = {'energy': 0,
                            'protein': 0,
                            'carbohydrates': 0,
                            'carbohydrates_sugar': 0,
                            'fat': 0,
                            'fat_saturated': 0,
                            'fibres': 0,
                            'sodium': 0}
        for meal in self.meal_set.select_related():
            for item in meal.mealitem_set.select_related():

                # Don't proceed if the ingredient was input by hand or has freetext units
                if item.ingredient and item.amount_gramm:
                    nutritional_info['energy'] += item.ingredient.energy * item.amount_gramm / 100
                    nutritional_info['protein'] += item.ingredient.protein * item.amount_gramm / 100
                    nutritional_info['carbohydrates'] += item.ingredient.carbohydrates * \
                        item.amount_gramm / 100

                    if item.ingredient.carbohydrates_sugar:
                        nutritional_info['carbohydrates_sugar'] += \
                            item.ingredient.carbohydrates_sugar * \
                            item.amount_gramm / 100

                    nutritional_info['fat'] += item.ingredient.fat * item.amount_gramm / 100
                    if item.ingredient.fat_saturated:
                        nutritional_info['fat_saturated'] += item.ingredient.fat_saturated * \
                            item.amount_gramm / 100

                    if item.ingredient.fibres:
                        nutritional_info['fibres'] += item.ingredient.fibres * \
                            item.amount_gramm / 100

                    if item.ingredient.sodium:
                        nutritional_info['sodium'] += item.ingredient.sodium * \
                            item.amount_gramm / 100
        return nutritional_info

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self


class Ingredient(models.Model):
    '''
    An ingredient, with some approximate nutrition values
    '''

    # Metaclass to set some other properties
    class Meta:
        ordering = ["name", ]

    language = models.ForeignKey(Language, verbose_name=_('Language'))

    name = models.CharField(max_length=200,
                            verbose_name=_('Name'),)

    energy = models.IntegerField(verbose_name=_('Energy'),
                                 help_text=_('In kcal per 100g'))

    protein = models.FloatField(verbose_name=_('Protein'),
                                help_text=_('In g per 100g of product'))

    carbohydrates = models.FloatField(verbose_name=_('Carbohydrates'),
                                      help_text=_('In g per 100g of product'))

    carbohydrates_sugar = models.FloatField(blank=True,
                                            null=True,
                                            verbose_name=_('Sugar content in carbohydrates'),
                                            help_text=_('In g per 100g of product'))

    fat = models.FloatField(blank=True,
                            verbose_name=_('Fat'),
                            help_text=_('In g per 100g of product'))

    fat_saturated = models.FloatField(blank=True,
                                      null=True,
                                      verbose_name=_('Saturated fat content in fats'),
                                      help_text=_('In g per 100g of product'))

    fibres = models.FloatField(blank=True,
                               null=True,
                               verbose_name=_('Fibres'),
                               help_text=_('In g per 100g of product'))

    sodium = models.FloatField(blank=True,
                               null=True,
                               verbose_name=_('Sodium'),
                               help_text=_('In g per 100g of product'))

    def get_absolute_url(self):
        '''
        Returns the canonical URL to view this object
        '''
        return reverse('wger.nutrition.views.ingredient_view', kwargs={'id': self.id,
                                                                       'slug': slugify(self.name)})

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "%s" % (self.name, )

    def get_owner_object(self):
        '''
        Ingredient has no owner information
        '''
        return False


class Meal(models.Model):
    '''
    A meal
    '''

    # Metaclass to set some other properties
    class Meta:
        ordering = ["time", ]

    plan = models.ForeignKey(NutritionPlan, verbose_name=_('Nutrition plan'))
    order = models.IntegerField(max_length=1, blank=True, verbose_name=_('Order'))
    time = models.TimeField(null=True, blank=True, verbose_name=_('Time (approx)'))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "%s Meal" % (self.order,)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self.plan


class MealItem(models.Model):
    '''
    An item (component) of a meal
    '''

    meal = models.ForeignKey(Meal, verbose_name=_('Nutrition plan'))
    order = models.IntegerField(max_length=1, blank=True, verbose_name=_('Order'))
    amount_gramm = models.IntegerField(max_length=4,
                                       blank=True,
                                       validators=[MaxValueValidator(1000)],
                                       verbose_name=_('Amount in gramms'))
    ingredient = models.ForeignKey(Ingredient, verbose_name=_('Ingredient'))

    def __unicode__(self):
        '''
        Return a more human-readable representation
        '''
        return "%sg ingredient %s" % (self.amount_gramm, self.ingredient_id)

    def get_owner_object(self):
        '''
        Returns the object that has owner information
        '''
        return self.meal.plan



from wger.manager.tests.index import *
from wger.manager.tests.demo_user import *
from wger.manager.tests.registration import *
from wger.manager.tests.preferences import *
from wger.manager.tests.change_password import *

from wger.manager.tests.workout import *
from wger.manager.tests.day import *
from wger.manager.tests.set import *
from wger.manager.tests.weight_log import *
from wger.manager.tests.copy_workout import *
from wger.manager.tests.pdf import *
